/*
 * @plotSymbol.java Version 1.1 07/09/2002
 *
 * Copyright (c) 2002 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

import plot.plotSymbolStruct;

/** Class plotSymbol
 *  <p> This method will create the different symbol types for plotting data
 *      values on a plot.
 *
 *  @version 1.1 07/09/2002
 *  @author  John R. Victorine
 */

public class plotSymbol extends Canvas implements MouseListener
{
  private int              iWidth  = 175;
  private int              iHeight = 300;
  private int              iSize   = 5;
  private int              iMax    = 2 * iSize;
  private plotSymbolStruct stPlot = new plotSymbolStruct();
  private JTextField       jTextField = null;

  /** Constructor plotSymbol()
   *  <p> This is the Constructor for this class
   */

  public plotSymbol()
  {
    stPlot = new plotSymbolStruct();

    stPlot.iEmpty  = stPlot.EMPTY;       // Open or Filled Symbol
    stPlot.iSymbol = stPlot.OVAL;        // Symbol Identifier.
    stPlot.iRed    = 0;                  // Red Color in RGB Setting
    stPlot.iGreen  = 0;                  // Green Color in RGB Setting
    stPlot.iBlue   = 0;                  // Blue Color in RGB Setting
    stPlot.symbol  = new String("Oval"); // Label for the selected symbol.

    setSize(iWidth, iHeight);
    setBackground(new Color(250, 250, 250));
  }

  /** Constructor plotSymbol()
   *  <p> This is the Constructor for this class
   * @param jTextField = The TextField to display the Text.
   */

  public plotSymbol(JTextField jTextField)
  {
    stPlot = new plotSymbolStruct();

    stPlot.iEmpty  = stPlot.EMPTY;       // Open or Filled Symbol
    stPlot.iSymbol = stPlot.OVAL;        // Symbol Identifier.
    stPlot.iRed    = 0;                  // Red Color in RGB Setting
    stPlot.iGreen  = 0;                  // Green Color in RGB Setting
    stPlot.iBlue   = 0;                  // Blue Color in RGB Setting
    stPlot.symbol  = new String("Oval"); // Label for the selected symbol.

    this.jTextField = jTextField;
    addMouseListener(this);

    setSize(iWidth, iHeight);
    setBackground(new Color(250, 250, 250));
  }

  /** Constructor plotSymbol()
   *  <p> This is the Constructor for this class
   *  @param stPlot = The Symbol Data Structure
   */

  public plotSymbol(plotSymbolStruct stPlot)
  {
    this.stPlot = stPlot;

    if ((stPlot.iWidth != 0) && (stPlot.iHeight != 0))
    {
      this.iWidth  = stPlot.iWidth;
      this.iHeight = stPlot.iHeight;
    }

    setSize(this.iWidth, this.iHeight);
    setBackground(new Color(250, 250, 250));
  }

  /** Constructor plotSymbol()
   *  <p> This is the Constructor for this class
   *  @param stPlot     = The Symbol Data Structure
   *  @param jTextField = The TextField to display the Text.
   */

  public plotSymbol(plotSymbolStruct stPlot, JTextField jTextField)
  {
    this.stPlot     = stPlot;
    this.jTextField = jTextField;

    addMouseListener(this);

    if ((stPlot.iWidth != 0) && (stPlot.iHeight != 0))
    {
      this.iWidth  = stPlot.iWidth;
      this.iHeight = stPlot.iHeight;
    }

    setSize(this.iWidth, this.iHeight);
    setBackground(new Color(250, 250, 250));
  }

  /** Method close()
   * <p> This method will close set the global variable to null in order to
   *     force Java to deallocate memory.
   */

  public void close()
  {
    stPlot     = null;
    jTextField = null;
  }

  /* ------------------------------------------------------------------------ *
   * --------------------------- Get Methods -------------------------------- *
   * ------------------------------------------------------------------------ */

  /** Method getLabel()
   *  <p> This method will retrieve a label of the Plot Symbol
   *  @param  iType   = Filled or Empty Symbol
   *  @param  iSymbol = Symbol Type
   *  @return sLabel  = The Text Label of the Symbol
   */

  public String getLabel(int iType, int iSymbol)
  {
    String sLabel = new String("");

    switch (iType)
    {
      case plot.plotSymbolStruct.EMPTY:
        sLabel = new String(plot.plotSymbolStruct.SHAPE_EMPTY[iSymbol]);
        break;

      case plot.plotSymbolStruct.FILLED:
        sLabel = new String(plot.plotSymbolStruct.SHAPE_FILLED[iSymbol]);
        break;
    }

    return (sLabel);
  }

  /** Method getPlotSymbolStructure()
   *  <p> This method will set the Symbol and the Color to be plotted.
   *
   * @param  iEmpty   = Open or Filled Symbol
   * @param  iSymbol  = Symbol Identifier.
   * @param  iRed     = Red Color in RGB Setting
   * @param  iGreen   = Green Color in RGB Setting
   * @param  iBlue    = Blue Color in RGB Setting
   * @param  symbol   = Label for the selected symbol.
   * @return stSymbol = The Plot Symbol Data Structure.
   */

  public static plotSymbolStruct getPlotSymbolStructure(int iEmpty,
                                                        int iSymbol,
                                                        int iRed,
                                                        int iGreen,
                                                        int iBlue,
                                                        String symbol)
  {
    plotSymbolStruct stSymbol = new plotSymbolStruct();

    stSymbol.iEmpty  = iEmpty;      // Open or Filled Symbol
    stSymbol.iSymbol = iSymbol;     // Symbol Identifier.
    stSymbol.iRed    = iRed;        // Red Color in RGB Setting
    stSymbol.iGreen  = iGreen;      // Green Color in RGB Setting
    stSymbol.iBlue   = iBlue;       // Blue Color in RGB Setting

    if (symbol != null)
      stSymbol.symbol  = new String(symbol); // Label for the selected symbol.

    return (stSymbol);
  }

  /* ------------------------------------------------------------------------ *
   * --------------------------- Set Methods -------------------------------- *
   * ------------------------------------------------------------------------ */

  /** Method setArea()
   *  <p> This method will allow the User to change the size of the plot area
   *  @param iWidth  = The Width of the Plot
   *  @param iHeight = The Height of the Plot
   */

  public void setArea(int iWidth, int iHeight)
  {
    this.iWidth  = iWidth;
    this.iHeight = iHeight;
  }

  /** Method setColor()
   *  <p> This method will allow the User to change the color of the Symbol
   *  @param iRed   = The Red Color
   *  @param iGreen = The Green Color
   *  @param iBlue  = The Blue Color
   */

  public void setColor(int iRed, int iGreen, int iBlue)
  {
    stPlot.iRed   = iRed;
    stPlot.iGreen = iGreen;
    stPlot.iBlue  = iBlue;
  }

  /** Method setSymbol()
   *  <p> This method will allow the User to change the Symbol type
   *  @param iEmpty  = The Symbol Empty or Filled.
   *  @param iSymbol = The Symbol ID
   *  @param symbol  = The Text to be used for Symbol
   */

  public void setSymbol(int iEmpty, int iSymbol, String symbol)
  {
    stPlot.iEmpty  = iEmpty;
    stPlot.iSymbol = iSymbol;
    stPlot.symbol  = new String(symbol);
  }

  /* ------------------------------------------------------------------------ *
   *  MouseListener Methods required when implementing the MouseListener Class.
   * ------------------------------------------------------------------------ */

  /** Method mouseMoved()
   *  <p> This method will retrieve the location of the mouse pointer and
   *       convert the postion to identify which symbol the user is selecting
   * @param event - Mouse Moved Event
   */

  public void mouseMoved(MouseEvent event) { }
  public void mouseDragged(MouseEvent event) { }
  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event) { }
  public void mousePressed(MouseEvent event) { }
  public void mouseReleased(MouseEvent event) { }
  public void mouseClicked(MouseEvent event)
  {
    int    iXm = event.getX();
    int    iYm = event.getY();
    int    iXo = 0;
    int    iYo = 0;

    if ((iXm > 0) && (iXm < iWidth) && (iYm > 0) && (iYm < iHeight))
    {
      for (int i=0; i<2; i++)
      {
        for (int j=0; j<7; j++)
        {
           int k = j + i * 7;
           iYo = 25 + k * 20;
           if ((iYm > iYo-iSize) && (iYm < iYo+iSize))
           {
             if (jTextField != null)
             {
               jTextField.setText(getLabel(i, j));
             }
           }
        }
      }
    }
  }

  /* ------------------------------------------------------------------------ *
   * ---------------------------- Draw Methods ------------------------------
   * ------------------------------------------------------------------------ */

  /** Method drawSymbol()
   *  <p> This method will draw a filled or empty symbol
   *  @param g       = Graphics Handle;
   *  @param stPlot  = Plot Symbol Data Structure
   *  @param iXp     = X Position
   *  @param iYp     = Y Position
   */

  public void drawSymbol(Graphics g, plotSymbolStruct stPlot, int iXp, int iYp)
  {
    int    i       = 0;
    int    iTotal  = 0;
    int    ix[]    = null;
    int    iy[]    = null;
    double dRadian = 0.0;

    g.setColor(new Color(stPlot.iRed, stPlot.iGreen, stPlot.iBlue));

    switch (stPlot.iEmpty)
    {
      case plot.plotSymbolStruct.EMPTY:
        switch (stPlot.iSymbol)
        {
          case plot.plotSymbolStruct.OVAL:
            g.drawOval(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.RECTANGLE:
            g.drawRect(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.TRIANGLE:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (90.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DIAMOND:
            iTotal = 4;
            ix = new int[4];
            iy = new int[4];
            for (i=0; i<4; i++)
            {
              dRadian = Math.PI * (0.0 + (i * 90.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DELTA:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (45.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.STAR:
            g.drawLine(iXp-iSize, iYp,   iXp+iSize, iYp);
            g.drawLine(iXp,   iYp-iSize, iXp,   iYp+iSize);
            g.drawLine(iXp-iSize, iYp-iSize, iXp+iSize, iYp+iSize);
            g.drawLine(iXp+iSize, iYp+iSize, iXp-iSize, iYp-iSize);
            break;
          case plot.plotSymbolStruct.HALF_MOON:
            g.drawArc(iXp-iSize, iYp-iSize, iMax, iMax, 90, 180);
            break;
        }
        break;

      case plot.plotSymbolStruct.FILLED:
        switch (stPlot.iSymbol)
        {
          case plot.plotSymbolStruct.OVAL:
            g.fillOval(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.RECTANGLE:
            g.fillRect(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.TRIANGLE:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (90.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DIAMOND:
            iTotal = 4;
            ix = new int[4];
            iy = new int[4];
            for (i=0; i<4; i++)
            {
              dRadian = Math.PI * (0.0 + (i * 90.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DELTA:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (45.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.STAR:
            g.drawLine(iXp-iSize, iYp,   iXp+iSize, iYp);
            g.drawLine(iXp,   iYp-iSize, iXp,   iYp+iSize);
            g.drawLine(iXp-iSize, iYp-iSize, iXp+iSize, iYp+iSize);
            g.drawLine(iXp+iSize, iYp+iSize, iXp-iSize, iYp-iSize);
            break;
          case plot.plotSymbolStruct.HALF_MOON:
            g.fillArc(iXp-iSize, iYp-iSize, iMax, iMax, 90, 180);
            break;
        }
        break;
    }
  }

  /** Method drawSymbol()
   *  <p> This method will draw a filled or empty symbol
   *  @param g       = Graphics Handle;
   *  @param iType   = Filled or Empty Symbol
   *  @param iSymbol = Symbol Type
   *  @param iXp     = X Position
   *  @param iYp     = Y Position
   */

  public void drawSymbol(Graphics g, int iType, int iSymbol, int iXp, int iYp)
  {
    int    i       = 0;
    int    iTotal  = 0;
    int    ix[]    = null;
    int    iy[]    = null;
    double dRadian = 0.0;

    g.setColor(new Color(stPlot.iRed, stPlot.iGreen, stPlot.iBlue));

    switch (iType)
    {
      case plot.plotSymbolStruct.EMPTY:
        switch (iSymbol)
        {
          case plot.plotSymbolStruct.OVAL:
            g.drawOval(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.RECTANGLE:
            g.drawRect(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.TRIANGLE:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (90.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DIAMOND:
            iTotal = 4;
            ix = new int[4];
            iy = new int[4];
            for (i=0; i<4; i++)
            {
              dRadian = Math.PI * (0.0 + (i * 90.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DELTA:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (45.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.drawPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.STAR:
            g.drawLine(iXp-iSize, iYp,   iXp+iSize, iYp);
            g.drawLine(iXp,   iYp-iSize, iXp,   iYp+iSize);
            g.drawLine(iXp-iSize, iYp-iSize, iXp+iSize, iYp+iSize);
            g.drawLine(iXp+iSize, iYp+iSize, iXp-iSize, iYp-iSize);
            break;
          case plot.plotSymbolStruct.HALF_MOON:
            g.drawArc(iXp-iSize, iYp-iSize, iMax, iMax, 90, 180);
            break;
        }
        break;

      case plot.plotSymbolStruct.FILLED:
        switch (iSymbol)
        {
          case plot.plotSymbolStruct.OVAL:
            g.fillOval(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.RECTANGLE:
            g.fillRect(iXp-iSize, iYp-iSize, iMax, iMax);
            break;
          case plot.plotSymbolStruct.TRIANGLE:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (90.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DIAMOND:
            iTotal = 4;
            ix = new int[4];
            iy = new int[4];
            for (i=0; i<4; i++)
            {
              dRadian = Math.PI * (0.0 + (i * 90.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.DELTA:
            iTotal = 3;
            ix = new int[3];
            iy = new int[3];
            for (i=0; i<3; i++)
            {
              dRadian = Math.PI * (45.0 + (i * 135.0)) / 180.0;
              ix[i] = iXp - (int) ( iSize * Math.sin(dRadian) );
              iy[i] = iYp - (int) ( iSize * Math.cos(dRadian) );
            }

            g.fillPolygon(ix, iy, iTotal);
            break;
          case plot.plotSymbolStruct.STAR:
            g.drawLine(iXp-iSize, iYp,   iXp+iSize, iYp);
            g.drawLine(iXp,   iYp-iSize, iXp,   iYp+iSize);
            g.drawLine(iXp-iSize, iYp-iSize, iXp+iSize, iYp+iSize);
            g.drawLine(iXp+iSize, iYp+iSize, iXp-iSize, iYp-iSize);
            break;
          case plot.plotSymbolStruct.HALF_MOON:
            g.fillArc(iXp-iSize, iYp-iSize, iMax, iMax, 90, 180);
            break;
        }
        break;
    }
  }

  /** Method drawLabel()
   *  <p> This method will draw a label of the Plot Symbol
   *  @param g       = Graphics Handle;
   *  @param iType   = Filled or Empty Symbol
   *  @param iSymbol = Symbol Type
   *  @param iXp     = X Position
   *  @param iYp     = Y Position
   */

  public void drawLabel(Graphics g, int iType, int iSymbol, int iXp, int iYp)
  {
    g.setColor(Color.black);

    g.setFont( new Font("Monospaced", Font.BOLD, 12) );

    switch (iType)
    {
      case plot.plotSymbolStruct.EMPTY:
        g.drawString(plot.plotSymbolStruct.SHAPE_EMPTY[iSymbol], iXp, iYp);
        break;

      case plot.plotSymbolStruct.FILLED:
        g.drawString(plot.plotSymbolStruct.SHAPE_FILLED[iSymbol], iXp, iYp);
        break;
    }
  }

  /** Method drawSymbols()
   *  <p> This method will draw the symbol list.
   *  @param g - Graphics Handle;
   */

  public void drawSymbols(Graphics g)
  {
    int iXg = 10;
    int iYg = 10;

    g.setColor(Color.black);
    g.drawString("AVAILABLE SYMBOLS", iXg, iYg);

    for (int i=0; i<2; i++)
    {
      for (int j=0; j<7; j++)
      {
        int k = j + i * 7;

        iXg = 10;
        iYg = 25 + k * 20;
        drawSymbol(g, i, j, iXg, iYg);
        drawLabel(g, i, j, iXg+20, iYg+5);
      }
    }
  }

  /** Method paint()
   *  <p> This method will paint the Scale data.
   *  @param g - Graphics Handle;
   */

  public void paint(Graphics g) { drawSymbols(g); }
}
