/*
 * @plotXYUtility.java Version 1.1 08/27/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import plot.plotXYListStruct;
import plot.plotXYStruct;
import plot.plotSymbolStruct;

/** Class plotXYUtility
 *  <p> This Class will provide basic utility for the data structures
 *
 *  @version 1.1 08/27/2015
 *  @author  John R. Victorine
 */

public class plotXYUtility
{
  /** Method copyList()
   * <p> This method will copy one Location Info structure to another
   * @param  stOld = The Location Information Data Structure
   * @return stNew = The Location Information Data Structure
   */

  public static plotXYListStruct copyList(plotXYListStruct stOld)
  {
    plotXYListStruct stNew = null;

    if (stOld != null)
    {
	  stNew        = new plotXYListStruct();
      stNew.iCount = stOld.iCount;
      stNew.stItem = new plotXYStruct[stNew.iCount];

      for (int i=0; i<stNew.iCount; i++)
      {
        stNew.stItem[i] = copy( stOld.stItem[i] );
	  }
	}

	return (stNew);
  }

  /** Method copy()
   * <p> This method will copy one Location Info structure to another
   * @param  stOld = The Location Information Data Structure
   * @return stNew = The Location Information Data Structure
   */

  public static plotXYStruct copy(plotXYStruct stOld)
  {
	int i=0;
    plotXYStruct stNew = null;

    if (stOld != null)
    {
	  stNew             = new plotXYStruct();

      stNew.iWidth      = stOld.iWidth;      // Width of the Plot Area
      stNew.iHeight     = stOld.iHeight;     // Height of the Plot Area

      stNew.iWo         = stOld.iWo;         // Width of the 2D Plot Area
      stNew.iHo         = stOld.iHo;         // Height of the 2D Plot Area

      // Titles

      stNew.sTitle1     = new String(stOld.sTitle1); // The First Line of the Title
      stNew.sTitle2     = new String(stOld.sTitle2); // The Second Line of the Title
      stNew.sTitle3     = new String(stOld.sTitle3); // The Third Line of the Title

      // Step Size

      stNew.dStep       = stOld.dStep;

      // X-Axis Values.

      stNew.iXLog       = stOld.iXLog;       // X-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iXCycles    = stOld.iXCycles;    // Number of Cycles
      stNew.dXMaximum   = stOld.dXMaximum;   // Maximum X-Axis
      stNew.dXMinimum   = stOld.dXMinimum;   // Minimum X-Axis
      stNew.dXIncrement = stOld.dXIncrement; // Increment along the X-Axis
      stNew.sXMnemonic  = new String(stOld.sXMnemonic); // X-Axis Mnemonic
      stNew.sXAxis      = new String(stOld.sXAxis);     // X-Axis Label.

      // Y-Axis Values.

      stNew.iYLog       = stOld.iYLog;       // Y-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iYCycles    = stOld.iYCycles;    // Number of Cycles
      stNew.dYMaximum   = stOld.dYMaximum;   // Maximum Y-Axis
      stNew.dYMinimum   = stOld.dYMinimum;   // Minimum Y-Axis
      stNew.dYIncrement = stOld.dYIncrement; // Increment along the Y-Axis
      stNew.sYMnemonic  = new String(stOld.sYMnemonic); // Y-Axis Mnemonic
      stNew.sYAxis      = new String(stOld.sYAxis);     // Y-Axis Label.

      // Z-Axis Values.

      stNew.iZLog       = stOld.iZLog;       // Z-Axis Semilog Indicator
                                             //   1-Semilog
                                             //   0-Linear
      stNew.iZCycles    = stOld.iZCycles;    // Number of Cycles
      stNew.dZMaximum   = stOld.dZMaximum;   // Maximum Z-Axis
      stNew.dZMinimum   = stOld.dZMinimum;   // Minimum Z-Axis
      stNew.dZIncrement = stOld.dZIncrement; // Increment along the Z-Axis
      stNew.sZMnemonic  = new String(stOld.sZMnemonic); // Z-Axis Mnemonic
      stNew.sZAxis      = new String(stOld.sZAxis);     // Z-Axis Label.

      // Plot Data.

      stNew.iRows       = stOld.iRows;             // Number of Rows
      stNew.iBins       = stOld.iBins;             // Number of Bins
      stNew.depths      = new double[stNew.iRows]; // Array of Depths
      stNew.dX          = new double[stNew.iRows]; // Array of Horizontal Axis Data
      stNew.dY          = new double[stNew.iRows]; // Array of Vertical Axis Data
      stNew.dZ          = new double[stNew.iRows]; // Array of Other Data
      stNew.sZ          = new String[stNew.iRows]; // Data descriptor array for filtering
      stNew.stSymbols   = new plotSymbolStruct[stNew.iRows]; // Array of Symbol & colors

      for (i=0; i<stNew.iRows; i++)
      {
		stNew.depths[i]    = stOld.depths[i];
		stNew.dX[i]        = stOld.dX[i];
		stNew.dY[i]        = stOld.dY[i];
		stNew.dZ[i]        = stOld.dZ[i];
		stNew.sZ[i]        = new String(stOld.sZ[i]);
        stNew.stSymbols[i] = plot.plotSymbol.getPlotSymbolStructure(
                               stOld.stSymbols[i].iEmpty,
                               stOld.stSymbols[i].iSymbol,
                               stOld.stSymbols[i].iRed,
                               stOld.stSymbols[i].iGreen,
                               stOld.stSymbols[i].iBlue,
                               stOld.stSymbols[i].symbol );
	  }

      // Legend

      stNew.sLegRange  = new String(stOld.sLegRange); // Depth Range
      stNew.sLegTitle  = new String(stOld.sLegTitle); // Legend Title
      stNew.bLegend    = stOld.bLegend;               // Legend OFF
      stNew.iLegend    = stOld.iLegend;               // Number of Legends
      stNew.stLegend   = new plotSymbolStruct[stNew.iLegend]; // Array of Symbols & colors

      for (i=0; i<stNew.iLegend; i++)
      {
        stNew.stLegend[i] = plot.plotSymbol.getPlotSymbolStructure(
                              stOld.stLegend[i].iEmpty,
                              stOld.stLegend[i].iSymbol,
                              stOld.stLegend[i].iRed,
                              stOld.stLegend[i].iGreen,
                              stOld.stLegend[i].iBlue,
                              stOld.stLegend[i].symbol );
	  }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static plotXYListStruct transfer(plotXYListStruct stOld)
  {
    plotXYListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }


  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static plotXYStruct transfer(plotXYStruct stOld)
  {
    plotXYStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copy( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method add()
   * <p> This method will add a data structure to a list structure
   * @param  st    = Data List Structure
   * @param  stNew = Data Structure to be added to list
   * @return st    = Data List Structure
   */

  public static plotXYListStruct add( plotXYListStruct st, plotXYStruct stNew)
  {
    int i=0;
    int iCount=0;
    int iFound=-1;

    plotXYListStruct stTemp = new plotXYListStruct();
    stTemp.stItem = new plotXYStruct[1];
    stTemp.iCount = 0;

    if (st != null)
    {
      stTemp.stItem = new plotXYStruct[st.iCount+1];
      for (i=0; i<st.iCount; i++)
      {
        stTemp.stItem[iCount] = new plotXYStruct();
        stTemp.stItem[iCount] = copy(st.stItem[i]);
        iCount++;
      }

      st.delete();
      st = null;
    }

    stTemp.stItem[iCount] = new plotXYStruct();
    stTemp.stItem[iCount] = copy(stNew);
    iCount++;

    st        = new plotXYListStruct();
    st.stItem = new plotXYStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<iCount; i++)
    {
      st.stItem[i] = new plotXYStruct();
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();
    stTemp = null;

    return (st);
  }
}

/*
 *  @version 1.1 08/27/2015
 *  @author  John Victorine
 */