/*
 * @quake3DPlotControlFrame.java Version 1.1 05/25/2011
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import gui.guiSearchDirectoryFrame;      // GUI Search Directory Frame

import iqstrat.iqstratHeadersListStruct; // Well Header List Structure

import quake.quakeSeisStruct;            // Quake Seismic Data Structure
import quake.quakePlotFilterListStruct;  // Plot Filter List Structure
import quake.quakeSensorsStruct;         // Sensor Data Structure

import plot.plotSymbolStruct;            // Symbol Data Structure
import plot.plotXYStruct;                // XY Data Structure
import plot.plotXYFrame;                 // XY Plot Frame
import plot.plotXYZFrame;                // XYZ Plot Frame
import plot.plotXYHistogramFrame;        // Histogram Plot Frame

import util.utilLandGridListStruct;      // TRS Land Grid List Structure

/** Class quake3DPlotControlFrame
 *  <p> This class will display the Cross Plot Control Frame
 *
 *  @version 1.1 05/25/2011
 *  @author  John R. Victorine
 */

public class quake3DPlotControlFrame extends JFrame implements ActionListener, Observer
{
  public final static int _MAGNITUDE = quake.quakePlotConstants._MAGNITUDE;
  public final static int _DATE      = quake.quakePlotConstants._DATE;
  public final static int _DEPTH     = quake.quakePlotConstants._DEPTH;

  public final static int _BOTH      = 0;
  public final static int _USGS      = 1;
  public final static int _KGS       = 2;

  private static final int _OFF      = plot.plotXYZData._OFF;
  private static final int _ON       = plot.plotXYZData._ON;

  private static final int START     = 0;
  private static final int END       = 1;
  private static final int MINIMUM   = 2;
  private static final int MAXIMUM   = 3;

  private static final int YEAR      = 0;
  private static final int MONTH     = 1;
  private static final int DAY       = 2;
  private static final int HOUR      = 3;
  private static final int MIN       = 4;
  private static final int SEC       = 5;

  // Input Variables

  private quakeSeisStruct          stData   = null; // Quake Seismic Data Structure
  private iqstratHeadersListStruct stWells  = null; // headers Information Data Structure
  private utilLandGridListStruct   stGrid   = null; // TRS Land Grid List Structure
  private quakeSensorsStruct       stSensor = null; // CO2 Seismic Sensor Locations Structure

  // Global Variables

  private Observable              notifier = null; // Observalbles Object

  private int                     iPanel   = _MAGNITUDE;
  private int                     iData    = _BOTH;

  // -- Create PNG & PDF documents

  private String                  sPath    = "";   // Directory Path for saving
  private String                  sName    = "";   // Filename for saving
  private guiSearchDirectoryFrame pSearch  = null; // Search for Directory

  // -- Plot Control Data

  private String                  sTitle1 = "Kansas Geological Survey";
  private String                  sTitle2 = "Kansas Earthquakes";
  private String                  sTitle3 = "";

  private String                  sXAxis  = "Longitude";
  private int                     iXLog   = 0;
  private double                  dXMin   = -98.35;
  private double                  dXMax   = -97.1; //-97.35;
  private double                  dXIncr  =   0.25;

  private String                  sYAxis  = "Latitude";
  private int                     iYLog   = 0;
  private double                  dYMin   = 37.0;
  private double                  dYMax   = 37.5;
  private double                  dYIncr  = 0.1;

  private String                  sZAxis  = "Depth (km)";
  private int                     iZLog   = 0;
  private double                  dZMin   = -15.0;
  private double                  dZMax   =   0.0;
  private double                  dZIncr  =   1.0;

  // -- Time Scale

  private Timestamp               tsStart = null; // Starting Date
  private Timestamp               tsEnd   = null; // Ending Date

  private Timestamp               tsSo    = null; // Original Starting Date
  private Timestamp               tsEo    = null; // Original Ending Date

  private int                     iDate   = -1;

  // -- Plot Classes

  private plotXYStruct            stPlotXY = null; // XY Plot Data Structure
  private plotXYFrame             plotXY   = null; // XY Plot Frame
  private plotXYZFrame            plotXYZ  = null; // XYZ Plot Frame
  private plotXYHistogramFrame    plotXYH  = null; // XY Plot Histogram Frame

  // -- Filter Data By

  private quakePlotFilterListStruct stLegend = null; // Legend Filter List Data Structure

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField txtTitle1  = new JTextField();
  private JTextField txtTitle2  = new JTextField();
  private JTextField txtTitle3  = new JTextField();

  // .. X-Axis Widgets

  private JTextField txtXAxis   = new JTextField();
  private JTextField txtXMin    = new JTextField();
  private JTextField txtXMax    = new JTextField();
  private JTextField txtXIncr   = new JTextField();

  // .. Y-Axis Widgets

  private JTextField txtYAxis   = new JTextField();
  private JTextField txtYMin    = new JTextField();
  private JTextField txtYMax    = new JTextField();
  private JTextField txtYIncr   = new JTextField();

  // .. Z-Axis Widgets

  private JTextField txtZAxis   = new JTextField();
  private JTextField txtZMin    = new JTextField();
  private JTextField txtZMax    = new JTextField();
  private JTextField txtZIncr   = new JTextField();

  // .. Start Date & Time

  private JTextField txtYYYYs   = new JTextField(); // Year Field
  private JTextField txtMMs     = new JTextField(); // Month Field
  private JTextField txtDDs     = new JTextField(); // Day Field

  private JTextField txtHHs     = new JTextField(); // 24 Hour Field
  private JTextField txtmms     = new JTextField(); // minute Field
  private JTextField txtssSs    = new JTextField(); // second Field

  // .... End Date & Time

  private JTextField txtYYYYe   = new JTextField(); // Year Field
  private JTextField txtMMe     = new JTextField(); // Month Field
  private JTextField txtDDe     = new JTextField(); // Day Field

  private JTextField txtHHe     = new JTextField(); // 24 Hour Field
  private JTextField txtmme     = new JTextField(); // minute Field
  private JTextField txtssSe    = new JTextField(); // second Field

  // .... Time Control Buttons

  private JButton    btnReset   = new JButton();    // Reset Button
  private JButton    btnModify  = new JButton();    // Modify Button

  // .. Filter By Data Source

  private JRadioButton rbBoth   = new JRadioButton();
  private JRadioButton rbUSGS   = new JRadioButton();
  private JRadioButton rbKGS    = new JRadioButton();

  // .. Filter By Buttons

  private JRadioButton rbMag    = new JRadioButton();
  private JRadioButton rbDepth  = new JRadioButton();
  private JRadioButton rbDate   = new JRadioButton();

  // .. Wells on or off

  private JRadioButton rbOFF    = new JRadioButton();
  private JRadioButton rbON     = new JRadioButton();

  // .. TRS on or off

  private JRadioButton rbTRSOFF = new JRadioButton();
  private JRadioButton rbTRSON  = new JRadioButton();

  // -- Menus
  // .. File Button Menu

  private JMenuItem mPDF  = null; // Create a PDF of Plot
  private JMenuItem mExit = null; // Exit Plot

  /** Constructor quake3DPlotControlFrame()
   *  <p> This is the constructor for this class
   *  @param stData   = Earthquake Data Structure
   *  @param stWells  = SWD Wells & Wellington KGS Wells
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public quake3DPlotControlFrame( quakeSeisStruct          stData,
                                  iqstratHeadersListStruct stWells,
                                  quakeSensorsStruct       stSensor,
                                  utilLandGridListStruct   stGrid )
  {
    try
    {
      this.stData   = stData;
      this.stWells  = stWells;
      this.stSensor = stSensor;
      this.stGrid   = stGrid;

	  if (stData != null)
	  {
		if ((stData.tsStart != null) && (stData.tsEnd != null))
		{
		  sTitle3 = new String(stData.tsStart.toString() + " to " + stData.tsEnd.toString());

          tsStart = new Timestamp(stData.tsStart.getTime()); // Starting Date
          tsEnd   = new Timestamp(stData.tsEnd.getTime());   // Ending Date

          tsSo    = new Timestamp(stData.tsStart.getTime()); // Starting Date
          tsEo    = new Timestamp(stData.tsEnd.getTime());   // Ending Date
		}
	  }

//      getLimits();

      jbInit();

      setDateTime(START, tsSo);
      setDateTime(END,   tsEo);

      setPlotStruct();
      display_plot();

      addWindowListener( new quake3DPlotControlFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel   pnlTop      = new JPanel();
    JPanel   pnlTitles   = new JPanel();
    JPanel   pnlCenter   = new JPanel();
    JPanel   pnlAxis     = new JPanel();
    JPanel   pnlXAxis    = new JPanel();
    JPanel   pnlYAxis    = new JPanel();
    JPanel   pnlZAxis    = new JPanel();
    JPanel   pnlTime     = new JPanel();
    JPanel   pnlBase     = new JPanel();
    JPanel   pnlBase1    = new JPanel();
    JPanel   pnlBase2    = new JPanel();
    JPanel   pnlBase3    = new JPanel();
    JPanel   pnlData     = new JPanel();
    JPanel   pnlFilter   = new JPanel();
    JPanel   pnlStatus   = new JPanel();
    JPanel   pnlTRS      = new JPanel();

    ButtonGroup groupD   = new ButtonGroup();
    ButtonGroup group    = new ButtonGroup();
    ButtonGroup groupS   = new ButtonGroup();
    ButtonGroup groupTRS = new ButtonGroup();

    TitledBorder titledBorderFilterBy = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Filter By:");
    titledBorderFilterBy.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderData = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Source:");
    titledBorderData.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderFilter = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Data:");
    titledBorderFilter.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStatus = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Wells:");
    titledBorderStatus.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTRS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "TRS Grid:");
    titledBorderTRS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    JMenuBar menuBar     = new JMenuBar();
    JMenu    menuFile    = new JMenu("File");

    // Build Menus

    // -- Menu Bar

    mPDF = new JMenuItem("Create PDF Document Plot");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.add( mExit );

    menuBar.add(menuFile);
    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new quake3DPlotControlFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( "Kansas Earthquake 3D Plot Control" );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Titles Panel

    pnlTitles = buildPlotTitlesPanel();

    // -- Top Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Axes Panel

    pnlAxis.setLayout(new GridLayout(3,1));

    pnlXAxis = buildPlotXAxisPanel();
    pnlYAxis = buildPlotYAxisPanel();
    pnlZAxis = buildPlotZAxisPanel();

    pnlTime  = BuildDateTimePanel();

    // Buttons Panel

    pnlBase.setLayout(new BorderLayout()); //new GridLayout(3,1));
    pnlBase.setBorder(titledBorderFilterBy);

    // -- Filter By Data Type Buttons

    pnlData.setLayout(new GridLayout(3,1));
    pnlData.setBorder(titledBorderData);

    rbBoth.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbBoth.setHorizontalAlignment(AbstractButton.CENTER);
    rbBoth.setSelected(true);
    rbBoth.setText("All Data   ");
    rbBoth.addActionListener(this);

    rbUSGS.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbUSGS.setHorizontalAlignment(AbstractButton.CENTER);
    rbUSGS.setSelected(true);
    rbUSGS.setText("USGS       ");
    rbUSGS.addActionListener(this);

    rbKGS.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbKGS.setHorizontalAlignment(AbstractButton.CENTER);
    rbKGS.setSelected(true);
    rbKGS.setText("CO2 Sensors");
    rbKGS.addActionListener(this);

    // -- Filter By Buttons

    pnlBase1.setLayout(new BorderLayout());

    pnlFilter.setLayout(new GridLayout(3,1));
    pnlFilter.setBorder(titledBorderFilter);

    rbMag.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbMag.setHorizontalAlignment(AbstractButton.CENTER);
    rbMag.setSelected(true);
    rbMag.setText("Magnitude  ");
    rbMag.addActionListener(this);

    rbDepth.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbDepth.setHorizontalAlignment(AbstractButton.CENTER);
    rbDepth.setText("Depth      ");
    rbDepth.addActionListener(this);

    rbDate.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbDate.setHorizontalAlignment(AbstractButton.CENTER);
    rbDate.setText("Date       ");
    rbDate.addActionListener(this);

    // Turn Well Status On or Off

    pnlBase2.setLayout(new BorderLayout());

    pnlStatus.setLayout(new GridLayout(2,1));
    pnlStatus.setBorder(titledBorderStatus);

    rbON.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbON.setHorizontalAlignment(AbstractButton.CENTER);
    rbON.setSelected(true);
    rbON.setText("On         ");
    rbON.addActionListener(this);

    rbOFF.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbOFF.setHorizontalAlignment(AbstractButton.CENTER);
    rbOFF.setText("Off        ");
    rbOFF.addActionListener(this);

    // TRS Grid On or Off

    pnlBase3.setLayout(new BorderLayout());

    pnlTRS.setLayout(new GridLayout(2,1));
    pnlTRS.setBorder(titledBorderTRS);

    rbTRSON.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbTRSON.setHorizontalAlignment(AbstractButton.CENTER);
    rbTRSON.setSelected(true);
    rbTRSON.setText("On         ");
    rbTRSON.addActionListener(this);

    rbTRSOFF.setFont(new java.awt.Font("Monospaced", 0, 11));
    rbTRSOFF.setHorizontalAlignment(AbstractButton.CENTER);
    rbTRSOFF.setText("Off        ");
    rbTRSOFF.addActionListener(this);

    // Attach Widgets

    this.getContentPane().add(pnlTop, BorderLayout.NORTH);
    pnlTop.add(pnlTitles,             BorderLayout.NORTH);

    pnlTop.add(pnlCenter,             BorderLayout.CENTER);
    pnlCenter.add(pnlAxis,            BorderLayout.CENTER);
    pnlAxis.add(pnlXAxis,             null);
    pnlAxis.add(pnlYAxis,             null);
    pnlAxis.add(pnlZAxis,             null);

    pnlCenter.add(pnlTime,            BorderLayout.SOUTH);

    pnlTop.add(pnlBase,               BorderLayout.EAST);
    pnlBase.add(pnlData,              BorderLayout.NORTH);
    pnlData.add(rbBoth,               null);
    pnlData.add(rbUSGS,               null);
    pnlData.add(rbKGS,                null);

    groupD.add(rbBoth);
    groupD.add(rbUSGS);
    groupD.add(rbKGS);

    pnlBase.add(pnlBase1,             BorderLayout.CENTER);
    pnlBase1.add(pnlFilter,           BorderLayout.NORTH);
    pnlFilter.add(rbMag,              null);
    pnlFilter.add(rbDepth,            null);
    pnlFilter.add(rbDate,             null);

    group.add(rbMag);
    group.add(rbDepth);
    group.add(rbDate);

    pnlBase1.add(pnlBase2,            BorderLayout.CENTER);
    pnlBase2.add(pnlStatus,           BorderLayout.NORTH);
    pnlStatus.add(rbON,               null);
    pnlStatus.add(rbOFF,              null);

    groupS.add(rbON);
    groupS.add(rbOFF);

    pnlBase2.add(pnlBase3,            BorderLayout.CENTER);
    pnlBase3.add(pnlTRS,              BorderLayout.NORTH);
    pnlTRS.add(rbTRSON,               null);
    pnlTRS.add(rbTRSOFF,              null);

    groupTRS.add(rbTRSON);
    groupTRS.add(rbTRSOFF);

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(450, 635));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation( (d.width - this.getSize().width)/2,
    this.setLocation( 10, 10);
//                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method buildPlotTitlesPanel()
   *  <p> This method will build the Plot Titles Panel
   */

  private JPanel buildPlotTitlesPanel()
  {
	JPanel panel     = new JPanel();
    JPanel pnlTitle1 = new JPanel();
    JPanel pnlTitle2 = new JPanel();
    JPanel pnlTitle3 = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Title:");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Title:");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Title:");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Titles Panel

    panel.setLayout(new GridLayout(3,1));

    // .. First Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText("");
    txtTitle1.setText(sTitle1);
    txtTitle1.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle1.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Second Title Panel

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText("");
    txtTitle2.setText(sTitle2);
    txtTitle2.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle2.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Third Title Panel

    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText("");
    txtTitle3.setText(sTitle3);
    txtTitle3.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle3.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

	// Attach Widgets

    panel.add(pnlTitle1,     null);
    pnlTitle1.add(txtTitle1, BorderLayout.CENTER);

    panel.add(pnlTitle2,     null);
    pnlTitle2.add(txtTitle2, BorderLayout.CENTER);

    panel.add(pnlTitle3,     null);
    pnlTitle3.add(txtTitle3, BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotXAxisPanel()
   *  <p> This method will build the Plot XAxis Panel
   */

  private JPanel buildPlotXAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlXAxis      = new JPanel();
    JPanel pnlXAxisRange = new JPanel();
    JPanel pnlXMin       = new JPanel();
    JPanel pnlXMax       = new JPanel();
    JPanel pnlXIncr      = new JPanel();

    TitledBorder titledBorderXAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

    txtXAxis.setText("");
    txtXAxis.setText(sXAxis);
    txtXAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtXAxis.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. X-Axis Range Panel

    pnlXAxisRange.setLayout(new GridLayout());

    // .. .. X-Axis Minimum Panel

    pnlXMin.setLayout(new BorderLayout());
    pnlXMin.setBorder(titledBorderMin);

    txtXMin.setText("");
    txtXMin.setText(""+dXMin);
    txtXMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMin.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. X-Axis Maximum Panel

    pnlXMax.setLayout(new BorderLayout());
    pnlXMax.setBorder(titledBorderMax);

    txtXMax.setText("");
    txtXMax.setText(""+dXMax);
    txtXMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMax.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. X-Axis Increment Panel

    pnlXIncr.setLayout(new BorderLayout());
    pnlXIncr.setBorder(titledBorderIncr);

    txtXIncr.setText("");
    txtXIncr.setText(""+dXIncr);
    txtXIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXIncr.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    panel.add(pnlXAxis,         BorderLayout.CENTER);
    pnlXAxis.add(txtXAxis,      BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisRange, BorderLayout.SOUTH);
    pnlXAxisRange.add(pnlXMin,  null);
    pnlXMin.add(txtXMin,        BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXMax,  null);
    pnlXMax.add(txtXMax,        BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXIncr, null);
    pnlXIncr.add(txtXIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotYAxisPanel()
   *  <p> This method will build the Plot YAxis Panel
   */

  private JPanel buildPlotYAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlYAxis      = new JPanel();
    JPanel pnlYAxisRange = new JPanel();
    JPanel pnlYMin       = new JPanel();
    JPanel pnlYMax       = new JPanel();
    JPanel pnlYIncr      = new JPanel();

    TitledBorder titledBorderYAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText("");
    txtYAxis.setText(sYAxis);
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
    pnlYMin.setBorder(titledBorderMin);

    txtYMin.setText("");
    txtYMin.setText(""+dYMin);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
    pnlYMax.setBorder(titledBorderMax);

    txtYMax.setText("");
    txtYMax.setText(""+dYMax);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
    pnlYIncr.setBorder(titledBorderIncr);

    txtYIncr.setText("");
    txtYIncr.setText(""+dYIncr);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    panel.add(pnlYAxis,         BorderLayout.CENTER);
    pnlYAxis.add(txtYAxis,      BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange, BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,  null);
    pnlYMin.add(txtYMin,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,  null);
    pnlYMax.add(txtYMax,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr, null);
    pnlYIncr.add(txtYIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /** Method buildPlotZAxisPanel()
   *  <p> This method will build the Plot ZAxis Panel
   */

  private JPanel buildPlotZAxisPanel()
  {
	JPanel panel         = new JPanel();
    JPanel pnlZAxis      = new JPanel();
    JPanel pnlZAxisRange = new JPanel();
    JPanel pnlZMin       = new JPanel();
    JPanel pnlZMax       = new JPanel();
    JPanel pnlZIncr      = new JPanel();

    TitledBorder titledBorderZAxis = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Z-Axis:");
    titledBorderZAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment:");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // Z-Axis Panel

    pnlZAxis.setLayout(new BorderLayout());
    pnlZAxis.setBorder(titledBorderZAxis);

    txtZAxis.setText("");
    txtZAxis.setText(sZAxis);
    txtZAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtZAxis.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Z-Axis Range Panel

    pnlZAxisRange.setLayout(new GridLayout());

    // .. .. Z-Axis Minimum Panel

    pnlZMin.setLayout(new BorderLayout());
    pnlZMin.setBorder(titledBorderMin);

    txtZMin.setText("");
    txtZMin.setText(""+dZMin);
    txtZMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZMin.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. Z-Axis Maximum Panel

    pnlZMax.setLayout(new BorderLayout());
    pnlZMax.setBorder(titledBorderMax);

    txtZMax.setText("");
    txtZMax.setText(""+dZMax);
    txtZMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZMax.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. .. Z-Axis Increment Panel

    pnlZIncr.setLayout(new BorderLayout());
    pnlZIncr.setBorder(titledBorderIncr);

    txtZIncr.setText("");
    txtZIncr.setText(""+dZIncr);
    txtZIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtZIncr.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    panel.add(pnlZAxis,         BorderLayout.CENTER);
    pnlZAxis.add(txtZAxis,      BorderLayout.NORTH);

    pnlZAxis.add(pnlZAxisRange, BorderLayout.SOUTH);
    pnlZAxisRange.add(pnlZMin,  null);
    pnlZMin.add(txtZMin,        BorderLayout.CENTER);

    pnlZAxisRange.add(pnlZMax,  null);
    pnlZMax.add(txtZMax,        BorderLayout.CENTER);

    pnlZAxisRange.add(pnlZIncr, null);
    pnlZIncr.add(txtZIncr,      BorderLayout.CENTER);

	return (panel);
  }

  /** Method BuildDateTimePanel()
   *  <p> This method will build the date/time panel on the control dialog
   */

  private JPanel BuildDateTimePanel()
  {
    JPanel pnlDateTime       = new JPanel();

    JPanel pnlDateTimeCenter = new JPanel();

    JPanel pnlDateTimeStart  = new JPanel();
    JPanel pnlDateTimeEnd    = new JPanel();

    JPanel pnlDateStart      = new JPanel();
    JPanel pnlYYYYs          = new JPanel();
    JPanel pnlMMs            = new JPanel();
    JPanel pnlDDs            = new JPanel();

    JPanel pnlTimeStart      = new JPanel();
    JPanel pnlHHs            = new JPanel();
    JPanel pnlmms            = new JPanel();
    JPanel pnlsss            = new JPanel();

    JPanel pnlDateEnd        = new JPanel();
    JPanel pnlYYYYe          = new JPanel();
    JPanel pnlMMe            = new JPanel();
    JPanel pnlDDe            = new JPanel();

    JPanel pnlTimeEnd        = new JPanel();
    JPanel pnlHHe            = new JPanel();
    JPanel pnlmme            = new JPanel();
    JPanel pnlsse            = new JPanel();

    JPanel pnlTimeButtons    = new JPanel();

    TitledBorder titledBorderDates = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Date & Time(24 Hour) Range");
    titledBorderDates.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderStartDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start Date Time");
    titledBorderStartDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEndDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End Date Time");
    titledBorderEndDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYYYY = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "YYYY");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMM = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "MM");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDD = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "DD");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHH = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "HH");
    titledBorderHH.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBordermm = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "mm");
    titledBordermm.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderssS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "ss.S");
    titledBorderssS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -------------------------------------------------------
    // ------------------- Date Time Panel -------------------
    // -------------------------------------------------------

    pnlDateTime.setLayout(new BorderLayout());
    pnlDateTime.setBorder(titledBorderDates);

    // -- Date Time Center Panel

    pnlDateTimeCenter.setLayout(new GridLayout(2,1));

    // .......................................................
    // ................... Date Time Start ...................
    // .......................................................

    pnlDateTimeStart.setLayout(new GridLayout());
    pnlDateTimeStart.setBorder(titledBorderStartDate);

    // -- Date Start Panel

    pnlDateStart.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYs.setLayout(new BorderLayout());
    pnlYYYYs.setBorder(titledBorderYYYY);
    pnlYYYYs.add(txtYYYYs,  BorderLayout.CENTER);

    txtYYYYs.setText("");
    txtYYYYs.setColumns(4);
	txtYYYYs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYs.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMs.setLayout(new BorderLayout());
    pnlMMs.setBorder(titledBorderMM);
    pnlMMs.add(txtMMs,  BorderLayout.CENTER);

    txtMMs.setText("");
    txtMMs.setColumns(2);
	txtMMs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMs.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDs.setLayout(new BorderLayout());
    pnlDDs.setBorder(titledBorderDD);
    pnlDDs.add(txtDDs,  BorderLayout.CENTER);

    txtDDs.setText("");
    txtDDs.setColumns(2);
	txtDDs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDs.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateStart.add(pnlYYYYs, BorderLayout.WEST);
    pnlDateStart.add(pnlMMs,   BorderLayout.CENTER);
    pnlDateStart.add(pnlDDs,   BorderLayout.EAST);

    // -- Time Start Panel

    pnlTimeStart.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHs.setLayout(new BorderLayout());
    pnlHHs.setBorder(titledBorderHH);
    pnlHHs.add(txtHHs, BorderLayout.CENTER);

    txtHHs.setText("");
    txtHHs.setColumns(2);
	txtHHs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHs.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmms.setLayout(new BorderLayout());
    pnlmms.setBorder(titledBordermm);
    pnlmms.add(txtmms, BorderLayout.CENTER);

    txtmms.setText("");
    txtmms.setColumns(2);
	txtmms.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmms.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsss.setLayout(new BorderLayout());
    pnlsss.setBorder(titledBorderssS);
    pnlsss.add(txtssSs, BorderLayout.CENTER);

    txtssSs.setText("");
    txtssSs.setColumns(4);
	txtssSs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSs.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeStart.add(pnlHHs, BorderLayout.WEST);
    pnlTimeStart.add(pnlmms, BorderLayout.CENTER);
    pnlTimeStart.add(pnlsss, BorderLayout.EAST);

    // .......................................................
    // .................... Date Time End ....................
    // .......................................................

    pnlDateTimeEnd.setLayout(new GridLayout());
    pnlDateTimeEnd.setBorder(titledBorderEndDate);

    // -- Date End Panel

    pnlDateEnd.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYe.setLayout(new BorderLayout());
    pnlYYYYe.setBorder(titledBorderYYYY);
    pnlYYYYe.add(txtYYYYe,  BorderLayout.CENTER);

    txtYYYYe.setText("");
    txtYYYYe.setColumns(4);
	txtYYYYe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYe.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMe.setLayout(new BorderLayout());
    pnlMMe.setBorder(titledBorderMM);
    pnlMMe.add(txtMMe,  BorderLayout.CENTER);

    txtMMe.setText("");
    txtMMe.setColumns(2);
	txtMMe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMe.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDe.setLayout(new BorderLayout());
    pnlDDe.setBorder(titledBorderDD);
    pnlDDe.add(txtDDe,  BorderLayout.CENTER);

    txtDDe.setText("");
    txtDDe.setColumns(2);
	txtDDe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDe.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateEnd.add(pnlYYYYe, BorderLayout.WEST);
    pnlDateEnd.add(pnlMMe,   BorderLayout.CENTER);
    pnlDateEnd.add(pnlDDe,   BorderLayout.EAST);

    // -- Time End Panel

    pnlTimeEnd.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHe.setLayout(new BorderLayout());
    pnlHHe.setBorder(titledBorderHH);
    pnlHHe.add(txtHHe, BorderLayout.CENTER);

    txtHHe.setText("");
    txtHHe.setColumns(2);
	txtHHe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHe.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmme.setLayout(new BorderLayout());
    pnlmme.setBorder(titledBordermm);
    pnlmme.add(txtmme, BorderLayout.CENTER);

    txtmme.setText("");
    txtmme.setColumns(2);
	txtmme.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmme.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsse.setLayout(new BorderLayout());
    pnlsse.setBorder(titledBorderssS);
    pnlsse.add(txtssSe, BorderLayout.CENTER);

    txtssSe.setText("");
    txtssSe.setColumns(4);
	txtssSe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSe.addFocusListener(new quake3DPlotControlFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeEnd.add(pnlHHe, BorderLayout.WEST);
    pnlTimeEnd.add(pnlmme, BorderLayout.CENTER);
    pnlTimeEnd.add(pnlsse, BorderLayout.EAST);

    // .......................................................
    // .................. Date Time Buttons ..................
    // .......................................................

    pnlTimeButtons.setLayout(new GridLayout());

    btnReset.setFont(new java.awt.Font("Dialog", 1, 11));
    btnReset.setPreferredSize(new Dimension(67, 20));
    btnReset.setText("Reset");
    btnReset.addActionListener(this);

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setPreferredSize(new Dimension(71, 20));
    btnModify.setText("Modify");
    btnModify.addActionListener(this);

    // Attach Widgets to Panel

    pnlDateTime.add(pnlDateTimeCenter,      BorderLayout.CENTER);
    pnlDateTimeCenter.add(pnlDateTimeStart, null);
    pnlDateTimeStart.add(pnlDateStart,      null);
    pnlDateTimeStart.add(pnlTimeStart,      null);

    pnlDateTimeCenter.add(pnlDateTimeEnd,   null);
    pnlDateTimeEnd.add(pnlDateEnd,          null);
    pnlDateTimeEnd.add(pnlTimeEnd,          null);

    pnlDateTime.add(pnlTimeButtons,         BorderLayout.SOUTH);
    pnlTimeButtons.add(btnReset,            null);
    pnlTimeButtons.add(btnModify,           null);

    return (pnlDateTime);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD quake3DPlotControlFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quake3DPlotControlFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    if (stWells != null)
      stWells.delete();
    stWells = null;

    if (stData != null)
      stData.delete();
    stData   = null;

    if (stSensor != null)
      stSensor.delete();
    stSensor = null;

    if (stGrid != null)
      stGrid.delete();
	stGrid   = null;

    notifier = null;

    sTitle1 = null;
    sTitle2 = null;
    sTitle3 = null;

    sXAxis  = null;
    sYAxis  = null;
    sZAxis  = null;

    // .. Time Scale

    tsStart   = null; // Starting Date
    tsEnd     = null; // Ending Date

    tsSo      = null; // Original Starting Date
    tsEo      = null; // Original Ending Date

    if (stPlotXY != null)
      stPlotXY.delete();
    stPlotXY   = null;

    if (plotXY != null)
      plotXY.close();
    plotXY = null;

    if (plotXYZ != null)
      plotXYZ.close();
    plotXYZ = null;

    if (plotXYH != null)
      plotXYH.close();
    plotXYH = null;

    sPath   = null;
    sName   = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    // Global Frame Widgets
    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;
    txtTitle3 = null;

    // .. X-Axis Widgets

    txtXAxis  = null;
    txtXMin   = null;
    txtXMax   = null;
    txtXIncr  = null;

    // .. Y-Axis Widgets

    txtYAxis  = null;
    txtYMin   = null;
    txtYMax   = null;
    txtYIncr  = null;

    // .. Z-Axis Widgets

    txtZAxis  = null;
    txtZMin   = null;
    txtZMax   = null;
    txtZIncr  = null;

    // -- Date & Time Widgets

    // .. Start Date & Time

    txtYYYYs    = null; // Year Field
    txtMMs      = null; // Month Field
    txtDDs      = null; // Day Field

    txtHHs      = null; // 24 Hour Field
    txtmms      = null; // minute Field
    txtssSs     = null; // second Field

    // .. End Date & Time

    txtYYYYe    = null; // Year Field
    txtMMe      = null; // Month Field
    txtDDe      = null; // Day Field

    txtHHe      = null; // 24 Hour Field
    txtmme      = null; // minute Field
    txtssSe     = null; // second Field

    // .. Time Control Buttons

    btnReset    = null;    // Reset Button
    btnModify   = null;    // Modify Button

    // .. Filter By Source Buttons

    rbBoth    = null;
    rbUSGS    = null;
    rbKGS     = null;

    // .. Filter By Buttons

    rbMag     = null;
    rbDepth   = null;
    rbDate    = null;

    rbON      = null;
    rbOFF     = null;

    rbTRSON   = null;
    rbTRSOFF  = null;

    // -- Menus

    mPDF      = null; // PDF
    mExit     = null; // Exit

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getMagnitudeLegend()
   *  <p> This method will set the data to Filter By Magnitude
   */

  private void getMagnitudeLegend()
  {
	int i=0;

	// Clear the Legend List Structure

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    // Get the Magnitude Legend List Structure

	stLegend = quake.quakePlotConstants.getFilter();

	// Clear the Plot Data Legend List Structure

    if (stPlotXY != null)
    {
      if (stPlotXY.iLegend > 0)
      {
        for (i=0; i<stPlotXY.iLegend; i++)
        {
    	  stPlotXY.stLegend[i].delete();
    	  stPlotXY.stLegend[i] = null;
  	    }
      }

      stPlotXY.stLegend = null;
      stPlotXY.iLegend = 0;
    }

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Magnitude:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

	// Build Plot Data Structure

    addData();

    if (plotXY != null)  { plotXY.setData( stPlotXY ); }
	if (plotXYZ != null) { plotXYZ.setData( stPlotXY ); }
	if (plotXYH != null) { plotXYH.setData( stPlotXY ); }
  }

  /** Method getDepthLegend()
   *  <p> This method will set the data to Filter By Depth
   */

  private void getDepthLegend()
  {
	int i=0;

	// Clear the Legend List Structure

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    // Get the Magnitude Legend List Structure

	stLegend = quake.quakePlotConstants.getFilter(
		         stPlotXY.dZMinimum, stPlotXY.dZMaximum, stPlotXY.dZIncrement);

	// Clear the Plot Data Legend List Structure

    if (stPlotXY != null)
    {
      if (stPlotXY.iLegend > 0)
      {
        for (i=0; i<stPlotXY.iLegend; i++)
        {
    	  stPlotXY.stLegend[i].delete();
    	  stPlotXY.stLegend[i] = null;
  	    }
      }

      stPlotXY.stLegend = null;
      stPlotXY.iLegend = 0;
    }

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Depth [km]:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

	// Build Plot Data Structure

    addData();

    if (plotXY  != null) { plotXY.setData( stPlotXY ); }
	if (plotXYZ != null) { plotXYZ.setData( stPlotXY ); }
	if (plotXYH != null) { plotXYH.setData( stPlotXY ); }
  }

  /** Method getDateLegend()
   *  <p> This method will set the data to Filter By Depth
   */

  private void getDateLegend()
  {
	int i=0;

	// Clear the Legend List Structure

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    // Get the Magnitude Legend List Structure

//	stLegend = quake.quakePlotConstants.getFilter( stData.tsStart, stData.tsEnd );
	stLegend = quake.quakePlotConstants.getFilter( tsStart, tsEnd );

	// Clear the Plot Data Legend List Structure

    if (stPlotXY != null)
    {
      if (stPlotXY.iLegend > 0)
      {
        for (i=0; i<stPlotXY.iLegend; i++)
        {
    	  stPlotXY.stLegend[i].delete();
    	  stPlotXY.stLegend[i] = null;
  	    }
      }

      stPlotXY.stLegend = null;
      stPlotXY.iLegend = 0;
    }

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Date [YYYY/MM]:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

	// Build Plot Data Structure

    addData();

    if (plotXY  != null) { plotXY.setData( stPlotXY ); }
	if (plotXYZ != null) { plotXYZ.setData( stPlotXY ); }
	if (plotXYH != null) { plotXYH.setData( stPlotXY ); }
  }

  /** Method getFilter()
   *  <p> This method will check if the data is part of the filter
   *  @param  depth  = the first depth
   *  @param  data   = the data value
   *  @return iValue = Legend
   */

  private int getFilter( double depth, double data )
  {
	int i,j;
	int    iValue = -1;
	double dBase  = 0.0;
	String sLithology = "";
	int    iLith      = 0;


	switch (iPanel)
	{
      case _MAGNITUDE:
      case _DEPTH:
      case _DATE:
        if (stLegend != null)
        {
          for (i=0; i<stLegend.iCount; i++)
          {
	        if ((data >= stLegend.stItem[i].dataStart) &&
	            (data < stLegend.stItem[i].dataEnd))
			{
  	          iValue = i;
		  	}
	      }
	    }
        break;
	}

	return (iValue);
  }

  /** Method isDepth()
   *  <p> This method will check if the data is part of the filter
   *  @param  depth  = the first depth
   *  @param  data   = the data value
   *  @return bValue = true, if the data within filter, false otherwise
   */

  private boolean isDepth( double depth, double data )
  {
	int i,j;
	boolean bValue     = false;
	double  dBase      = 0.0;
	String  sLithology = "";
	int     iLith      = 0;

    if ((depth >= stPlotXY.dZMinimum) && (depth <= stPlotXY.dZMaximum))
    {
  	  switch (iPanel)
	  {
        case _MAGNITUDE:
        case _DEPTH:
        case _DATE:
          if (stLegend != null)
          {
            for (i=0; i<stLegend.iCount; i++)
            {
			  if ((data >= stLegend.stItem[i].dataStart) &&
			      (data < stLegend.stItem[i].dataEnd))
			  {
  	            bValue = true;
		  	  }
		    }
	      }
          break;
	  }
	}

	return (bValue);
  }

  /** Method isData()
   *  <p> This method will check if the data is part of the filter
   *  @param  src    = source of data
   *  @return bValue = true, if the data within filter, false otherwise
   */

  private boolean isData( String src )
  {
	boolean bValue     = false;

	switch (iData)
	{
      case _BOTH:
        bValue = true;
        break;

      case _USGS:
        if (src.equals("USGS"))
          bValue = true;
        break;

      case _KGS:
        if (src.equals("CO2 Sensor Array"))
          bValue = true;
        break;
    }

	return (bValue);
  }

  /** Method isDate()
   *  <p> This method will check if the data is part of the filter
   *  @param  src    = source of data
   *  @return bValue = true, if the data within filter, false otherwise
   */

  private boolean isDate( Timestamp ts )
  {
	boolean bValue     = false;

    if (ts != null)
    {
      long  lstart = tsStart.getTime();
      long  lend   = tsEnd.getTime();
      long  ldata  = ts.getTime();

      if ((ldata >= lstart) && (ldata <= lend))
        bValue = true;
	}

	return (bValue);
  }

  /** Method getLimits()
   *  <p> This method will determine the limits of the earthquake data
   */

  private void getLimits()
  {
    double dXMin   = -98.35;
    double dXMax   = -97.145;
    double dXIncr  = 1.215;

    double dYMin   = 37.0;
    double dYMax   = 37.5;
    double dYIncr  = 0.25;

    double dZMin   = -10.0;
    double dZMax   =   0.0;
    double dZIncr  =   1.0;

    if (stData != null)
    {
	  for (int i=0; i<stData.iRows; i++)
	  {
		if (i==0)
		{
		  dXMin = stData.dLongitude[i];
		  dXMax = stData.dLongitude[i];

		  dYMin = stData.dLatitude[i];
		  dYMax = stData.dLatitude[i];

		  dZMin = stData.depth[i];
		  dZMax = stData.depth[i];
		}

		if (dXMin > stData.dLongitude[i])
		  dXMin = stData.dLongitude[i];

		if (dXMax < stData.dLongitude[i])
		  dXMax = stData.dLongitude[i];

		if (dYMin > stData.dLatitude[i])
		  dYMin = stData.dLatitude[i];

		if (dYMax < stData.dLatitude[i])
		  dYMax = stData.dLatitude[i];

		if (dZMin > stData.depth[i])
		  dZMin = stData.depth[i];

		if (dZMax < stData.depth[i])
		  dZMax = stData.depth[i];
	  }

      dXIncr = (double)( (int) (100.0 * (dXMax - dXMin)) / 1000.0 );
      dYIncr = (double)( (int) (100.0 * (dYMax - dYMin)) / 1000.0 );
      dZIncr = (double)( (int) (100.0 * (dZMax - dZMin)) / 1000.0 );

      System.out.println("X: " + dXMin+" "+dXMax+" "+dXIncr);
      System.out.println("Y: " + dYMin+" "+dYMax+" "+dYIncr);
      System.out.println("Z: " + dZMin+" "+dZMax+" "+dZIncr);
	}
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setDateTime()
   *  <p> This method will set the Date and Time Value into either the
   *      Start Date Time or End Date Time.
   *  @param iDate = 0 for Start, 1 for End
   *  @param ts    = timestamp
   */

  private void setDateTime(int iDate, Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String sDelimiter2 = new String("[:]+");
    String str         = "";
    String sDate       = "";
    String sTime       = "";
    String date[]      = { "", "", "" };
    String time[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

	    iContinue = 1;

        if (iContinue == 1)
	    {
	      iContinue = 0;

	      str = new String(ts.toString());
	      tokens = str.split( sDelimiter );
	      if (tokens.length > 0)
	        sTime = new String(tokens[1].trim());

          tokens = sTime.split( sDelimiter2 );
          if (tokens.length == 3)
          {
  	        for (i=0; i<3; i++)
  	        {
	          time[i] = new String(tokens[i]);
	        }

	        iContinue = 1;

	        if (iContinue == 1)
	        {
	          switch (iDate)
	          {
	            case START:
	              txtYYYYs.setText(date[0]);
	              txtMMs.setText(date[1]);
	              txtDDs.setText(date[2]);

	              txtHHs.setText(time[0]);
	              txtmms.setText(time[1]);
	              txtssSs.setText(time[2]);
	              break;

	            case END:
	              txtYYYYe.setText(date[0]);
	              txtMMe.setText(date[1]);
	              txtDDe.setText(date[2]);

	              txtHHe.setText(time[0]);
	              txtmme.setText(time[1]);
	              txtssSe.setText(time[2]);
	              break;
	          }
	        }
  	      }
	    }
      }
    }
  }

  /** Method setPlotStruct()
   * <p> This method will set the Plot Structure
   */

  private void setPlotStruct()
  {
	int i=0;

	stPlotXY             = new plotXYStruct();

    stPlotXY.sTitle1     = new String( sTitle1 );
    stPlotXY.sTitle2     = new String( sTitle2 );
    stPlotXY.sTitle3     = new String( sTitle3 );

    stPlotXY.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotXY.iXCycles    = 1;      // Number of Cycles
    stPlotXY.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotXY.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotXY.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotXY.sXMnemonic  = new String( "" );
    stPlotXY.sXAxis      = new String( sXAxis );

    stPlotXY.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotXY.iYCycles    = 1;      // Number of Cycles
    stPlotXY.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotXY.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotXY.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotXY.sYMnemonic  = new String( "" );
    stPlotXY.sYAxis      = new String( sYAxis );

    stPlotXY.iZLog       = 0;      // Z-Axis Semilog Indicator
    stPlotXY.iZCycles    = 1;      // Number of Cycles
    stPlotXY.dZMaximum   = dZMax;  // Maximum Z-Axis
    stPlotXY.dZMinimum   = dZMin;  // Minimum Z-Axis
    stPlotXY.dZIncrement = dZIncr; // Increment along the Z-Axis
    stPlotXY.sZMnemonic  = new String( "" );
    stPlotXY.sZAxis      = new String( sZAxis );

    stLegend = quake.quakePlotConstants.getFilter();

    // Add Legend to Plot Data Structure

    if (stLegend != null)
    {
	  if (stPlotXY != null)
	  {
  		if (stLegend.iCount > 0)
  		{
          stPlotXY.bLegend   = true;
          stPlotXY.sLegTitle = new String( "Magnitude:" );
		  stPlotXY.stLegend  = new plotSymbolStruct[stLegend.iCount];
		  stPlotXY.iLegend   = stLegend.iCount;

		  for (i=0; i<stLegend.iCount; i++)
		  {
            stPlotXY.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stLegend.stItem[i].iSymbol,
                    stLegend.stItem[i].iRed,
                    stLegend.stItem[i].iGreen,
                    stLegend.stItem[i].iBlue,
                    stLegend.stItem[i].symbol );
	      }
		}
	  }
    }

    addData();
  }

  /** Method addData()
   * <p> This method will add the plot data
   */

  private void addData()
  {
    int    i        = 0;
    int    iRed     = 0;
    int    iGreen   = 0;
    int    iBlue    = 0;
    double dLog     = 0.0;
    int    iCount   = 0;
    int    iFilter  = -1;
    double dOut[][] = null;
    double data     = 0.0;
    double data1    = 0.0;
	int    iout[]   = { 0, 0 };
    int    iYear    = 0;
    int    iMonth   = 0;
    int    iSymbol  = -1;
    int    iArray   = -1;
    int    iContinue = 0;
    double d1        = 0.0;
    int    icnt      = 0;
    int    iDepth    = -1;

    // Clear the Plot Data Structure

    if (stPlotXY != null)
    {
      stPlotXY.dX = null; // Array of Horizontal Axis Data
      stPlotXY.dY = null; // Array of Vertical Axis Data
      stPlotXY.dZ = null; // Array of Other Data
      stPlotXY.sZ = null; // Array of Description

      if (stPlotXY.stSymbols != null)
      {
	    for(i=0; i<stPlotXY.iRows; i++)
	    {
		  if (stPlotXY.stSymbols[i] != null)
		  {
		    stPlotXY.stSymbols[i].delete();
	    	stPlotXY.stSymbols[i] = null;
	      }
	    }
        stPlotXY.stSymbols = null;
      }
	}

    if (stData != null)
    {
      for (i=0; i<stData.iRows; i++)
      {
        switch (iPanel)
	    {
          case _MAGNITUDE:
            data = stData.mag[i];
            break;

          case _DATE:
            iout   = quake.quakePlotConstants.setDate( stData.ts[i] );
            iYear  = iout[0];
            iMonth = iout[1];
            data   = (double) (iYear * 100  + iMonth);
//System.out.println(data);
//if (data > d1)
//{
//System.out.println(d1+" "+icnt);
//  d1   = data;
//  icnt = 0;
//}
//icnt++;
            break;

          case _DEPTH:
            data = stData.depth[i];
            break;
	    }

//System.out.println(isDepth( stData.depth[i], data )+" "+ stData.depth[i] +" "+data);
        if (isDate(stData.ts[i]))
        {
          if (isData(stData.src[i]))
          {
            if (isDepth( stData.depth[i], data ))
            {
              iCount++;
			}
          }
		}
	  }

      stPlotXY.iRows  = iCount;             // Number of Rows
      stPlotXY.depths = new double[iCount]; // Array of Depths
      stPlotXY.dX     = new double[iCount]; // Array of Horizontal Axis Data
      stPlotXY.dY     = new double[iCount]; // Array of Vertical Axis Data
      stPlotXY.dZ     = new double[iCount]; // Array of Other Data
      stPlotXY.sZ     = new String[iCount]; // Array of Description
      stPlotXY.stSymbols = new plotSymbolStruct[iCount]; // Array of Symbol

      iCount = 0;

      for (i=0; i<stData.iRows; i++)
      {
//		iDepth = -1;
//		if (stData.depth[i] <= -5)
//		  iDepth = 1;

        iDepth = isGreaterThan5km(stData.depth[i], stData.dLatitude[i], stData.dLongitude[i]);

        switch (iPanel)
	    {
          case _MAGNITUDE:
            data = stData.mag[i];
            break;

          case _DATE:
            iout   = quake.quakePlotConstants.setDate( stData.ts[i] );
            iYear  = iout[0];
            iMonth = iout[1];
            data   = (double) (iYear * 100  + iMonth);
            break;

          case _DEPTH:
            data = stData.depth[i];
            break;
	    }

        iContinue = 0;
        if (isDate(stData.ts[i]))
        {
          if (isData(stData.src[i]))
          {
            if (isDepth( stData.depth[i], data ))
            {
              iContinue = 1;
			}
          }
		}

        if (iContinue == 1)
        {
	      if (iCount < stPlotXY.iRows)
	      {
			stPlotXY.depths[iCount] = stData.depth[i];
            stPlotXY.dZ[iCount]     = stData.depth[i];
            stPlotXY.dY[iCount]     = stData.dLatitude[i];
            stPlotXY.dX[iCount]     = stData.dLongitude[i];
            stPlotXY.sZ[iCount]     = new String("");

            if (stData.src[i].equals("CO2 Sensor Array"))
              iArray = 1;
            else
              iArray = -1;

            iFilter = getFilter( stData.depth[i], data );
            if (iFilter == 999)
            {
              stPlotXY.stSymbols[iCount] =
                plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.OVAL,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
            }
            else
            {
   			  stPlotXY.sZ[iCount] = new String(
			    stLegend.stItem[iFilter].symbol );

			  if (iArray == -1)
			  {
				stPlotXY.stSymbols[iCount] =
		          plot.plotSymbol.getPlotSymbolStructure(
                      plot.plotSymbolStruct.FILLED,
                      stLegend.stItem[iFilter].iSymbol,
                      stLegend.stItem[iFilter].iRed,
                      stLegend.stItem[iFilter].iGreen,
                      stLegend.stItem[iFilter].iBlue,
                      stLegend.stItem[iFilter].symbol);
			  }
			  else
			  {
				if (iDepth == -1)
				{
				  stPlotXY.stSymbols[iCount] =
		            plot.plotSymbol.getPlotSymbolStructure(
                        plot.plotSymbolStruct.FILLED,
                        plot.plotSymbolStruct.TRIANGLE,
                        stLegend.stItem[iFilter].iRed,
                        stLegend.stItem[iFilter].iGreen,
                        stLegend.stItem[iFilter].iBlue,
                        stLegend.stItem[iFilter].symbol);
			    }
			    else
			    {
				  stPlotXY.stSymbols[iCount] =
		            plot.plotSymbol.getPlotSymbolStructure(
                        plot.plotSymbolStruct.EMPTY,
                        plot.plotSymbolStruct.TRIANGLE,
                        stLegend.stItem[iFilter].iRed,
                        stLegend.stItem[iFilter].iGreen,
                        stLegend.stItem[iFilter].iBlue,
                        stLegend.stItem[iFilter].symbol);
				}
			  }
		    }

            iCount++;
		  }
		}
	  }
	}
  }

  /** Method isGreaterThan5km()
   *  <p> This method will determine if the distance between Wellington KGS 2-32
   *      and seismic is greater than or equal to 5 km
   *
   *        Wellington KGS 2-32
   *           Longitude: -97.4418
   *           Latitude:   37.3105345
   *
   *       Finding distances based on Latitude and Longitude
   *           { http://andrew.hedges.name/experiments/haversine/ }
   *           "This JavaScript uses the Haversine Formula (shown below) expressed
   *            in terms of a two-argument inverse tangent function to calculate the
   *            great circle distance between two points on the Earth. This is the
   *            method recommended for calculating short distances by Bob Chamberlain
   *            (rgc@jpl.nasa.gov) of Caltech and NASA's Jet Propulsion Laboratory as
   *            described on the U.S. Census Bureau Web site."
   *
   *       dlon = lon2 - lon1
   *       dlat = lat2 - lat1
   *       a = (sin(dlat/2))^2 + cos(lat1) * cos(lat2) * (sin(dlon/2))^2
   *       c = 2 * atan2( sqrt(a), sqrt(1-a) )
   *       d = R * c (where R is the radius of the Earth)
   *
   *  @param  depth = depth of earthquake
   *  @param  dLat  = Latitude location of earthquake
   *  @param  dLong = Longitude location of earthquake
   *  @return iKM   = indicator that the distance is greater than or equal to 5 km (1) or not (-1)
   */

  private int isGreaterThan5km(double depth, double dLat, double dLong)
  {
	int iKM = -1;
	double d232Lat  = 37.3105345;
	double d232Long = -97.4418;
	double R        = 6373.0; // Radius of the Earth (km)
	double delLat   = 0.0;
    double delLong  = 0.0;
    double a        = 0.0;
    double c        = 0.0;
    double d        = 0.0;
    double depth1   = Math.abs(depth);

    delLong = Math.PI * (dLong - d232Long) / 180.0;
    delLat  = Math.PI * (dLat  - d232Lat) / 180.0;

    a = Math.sin(delLat/2) * Math.sin(delLat/2) +
        Math.cos(d232Lat) * Math.cos(dLat) * Math.sin(delLong/2) * Math.sin(delLong/2);

    c = 2.0 * Math.atan2( Math.sqrt(a), Math.sqrt(1.0-a) );
    d = Math.sqrt( (R*c)*(R*c) + depth1*depth1 );

    if (d >= 5)
      iKM = 1;

	return (iKM);
  }

  /** Method modifyTime()
   *  <p> This method will set the Start and End Time
   */

  private void modifyTime()
  {
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;

    sText   = new String( txtYYYYs.getText() + "-" +
	                      txtMMs.getText()   + "-" +
	                      txtDDs.getText()   + " " +
	                      txtHHs.getText()   + ":" +
	                      txtmms.getText()   + ":" +
	                      txtssSs.getText() );

    try
    {
      ltime   = dfm.parse(sText).getTime();
      tsStart = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

    sText   = new String( txtYYYYe.getText() + "-" +
	                      txtMMe.getText()   + "-" +
	                      txtDDe.getText()   + " " +
	                      txtHHe.getText()   + ":" +
	                      txtmme.getText()   + ":" +
	                      txtssSe.getText() );

    try
    {
      ltime   = dfm.parse(sText).getTime();
      tsEnd   = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

    sTitle3 = new String(tsStart.toString() + " to " + tsEnd.toString());
    txtTitle3.setText(sTitle3);
    stPlotXY.sTitle3 = new String(sTitle3);

    setFilter();
  }

  /** Method setFilter()
   *  <p> This method will set the filter type
   */

  private void setFilter()
  {
    switch (iPanel)
    {
  	  case _MAGNITUDE:
	    getMagnitudeLegend();
	    break;
	  case _DEPTH:
	    getDepthLegend();
	    break;
	  case _DATE:
	    getDateLegend();
	    break;
	}
  }

  /* ===================================================================== *
   * ------------------------- PLOT METHODS ------------------------------ *
   * ===================================================================== */

  /** Method display_plot()
   *  <p> This method will create the 3D Plot
   */

  private void display_plot()
  {
    plotXY  = new plotXYFrame( notifier, 0, stPlotXY, stWells, stSensor, stGrid );
    plotXYZ = new plotXYZFrame( notifier, 0, stPlotXY, stWells, stSensor, stGrid );
    plotXYH = new plotXYHistogramFrame( notifier, 0, stPlotXY, "3D" );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void createPDF()
  {
    if (plotXY != null)
    {
      quake.io.quakeIO_Files.createImage( sPath, sName+"_2D", plotXY.getImage() );
      quake.io.quakeIO_Files.createHTML( sPath, sName+"_2D" );
    }

    if (plotXYZ != null)
    {
      quake.io.quakeIO_Files.createImage( sPath, sName+"_3D", plotXYZ.getImage() );
      quake.io.quakeIO_Files.createHTML( sPath, sName+"_3D" );
    }

    if (plotXYH != null)
    {
      quake.io.quakeIO_Files.createImage( sPath, sName+"_HIST", plotXYH.getImage() );
      quake.io.quakeIO_Files.createHTML( sPath, sName+"_HIST" );
	}
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnReset)    // Reset Button
    {
      setDateTime(START, tsSo);
      setDateTime(END,   tsEo);

	  modifyTime();
	}

    if (event.getSource() == btnModify)    // Modify Button
    {
	  modifyTime();
	}

    if (event.getSource() == rbBoth)
    {
	  iData = _BOTH;
	  setFilter();
	}

    if (event.getSource() == rbUSGS)
    {
	  iData = _USGS;
	  setFilter();
	}

    if (event.getSource() == rbKGS)
    {
	  iData = _KGS;
	  setFilter();
	}

    if (event.getSource() == rbMag)
    {
	  iPanel = _MAGNITUDE;
	  getMagnitudeLegend();
	}

    if (event.getSource() == rbDepth)
    {
	  iPanel = _DEPTH;
	  getDepthLegend();
	}
    if (event.getSource() == rbDate)
    {
      iPanel = _DATE;
      getDateLegend();
	}

    if (event.getSource() == rbON )
    {
	  plotXY.setStatus(_ON);
	  plotXYZ.setStatus(_ON);
	}

    if (event.getSource() == rbOFF)
    {
	  plotXY.setStatus(_OFF);
	  plotXYZ.setStatus(_OFF);
	}

    if (event.getSource() == rbTRSON )
    {
	  plotXY.setGrid(_ON);
	  plotXYZ.setGrid(_ON);
	}

    if (event.getSource() == rbTRSOFF)
    {
	  plotXY.setGrid(_OFF);
	  plotXYZ.setGrid(_OFF);
	}

    if (event.getSource() == mPDF)
    {
	  if (sPath.length() == 0)
        sPath = quake.io.quakeIO_Files.getHomeDir();
      sName   = new String( "KS_Earthquake" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
	}

    // Exit Panel

	if (event.getSource() == mExit)
	{
	  close();
	}
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if (e.getSource() == txtTitle1)
    {
      stPlotXY.sTitle1 = new String( txtTitle1.getText() );
	  if (plotXY  != null) { plotXY.setTitle1( stPlotXY.sTitle1 ); }
	  if (plotXYZ != null) { plotXYZ.setTitle1( stPlotXY.sTitle1 ); }
	  if (plotXYH != null) { plotXYH.setTitle1( stPlotXY.sTitle1 ); }
	}
    if (e.getSource() == txtTitle2)
    {
      stPlotXY.sTitle2 = new String( txtTitle2.getText() );
	  if (plotXY  != null) { plotXY.setTitle2( stPlotXY.sTitle2 ); }
	  if (plotXYZ != null) { plotXYZ.setTitle2( stPlotXY.sTitle2 ); }
	  if (plotXYH != null) { plotXYH.setTitle2( stPlotXY.sTitle2 ); }
	}
    if (e.getSource() == txtTitle3)
    {
      stPlotXY.sTitle3 = new String( txtTitle3.getText() );
	  if (plotXY  != null) { plotXY.setTitle3( stPlotXY.sTitle3 ); }
	  if (plotXYZ != null) { plotXYZ.setTitle3( stPlotXY.sTitle3 ); }
	  if (plotXYH != null) { plotXYH.setTitle3( stPlotXY.sTitle3 ); }
	}

    // .. X-Axis Widgets

    if (e.getSource() == txtXAxis)
    {
      stPlotXY.sXAxis = new String( txtXAxis.getText() );

      if (plotXY != null)
      {
        plotXY.setXAxis(  stPlotXY.sXAxis,
                          stPlotXY.iXLog,
	                      stPlotXY.dXMaximum,
	                      stPlotXY.dXMinimum,
	                      stPlotXY.dXIncrement,
                          stPlotXY.iXCycles );
      }

      if (plotXYZ != null)
      {
        plotXYZ.setXAxis( stPlotXY.sXAxis,
                          stPlotXY.iXLog,
	                      stPlotXY.dXMaximum,
	                      stPlotXY.dXMinimum,
	                      stPlotXY.dXIncrement,
                          stPlotXY.iXCycles );
      }

      if (plotXYH != null)
      {
        plotXYH.setXAxis( stPlotXY.sXAxis,
                          stPlotXY.iXLog,
	                      stPlotXY.dXMaximum,
	                      stPlotXY.dXMinimum,
	                      stPlotXY.dXIncrement,
                          stPlotXY.iXCycles );
      }
	}
    if (e.getSource() == txtXMin )
    {
      iNumeric = 1;
      sTemp = txtXMin.getText();
      sMessage = new String("X-Minimum Value is a Numeric Field");

      s1 = txtXMin.getText();
      s2 = txtXMax.getText();

      if (s1.equals(s2))
      {
		txtXMin.setText(""+stPlotXY.dXMinimum);
//		txtXMax.setText(""+stPlotXY.dXMaximum);

		sTemp = new String("A");
        sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtXMin.setText(""+stPlotXY.dXMinimum);

		  sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtXMax )
    {
      iNumeric = 1;
      sTemp = txtXMax.getText();
      sMessage = new String("X-Maximum Value is a Numeric Field");

      s1 = txtXMin.getText();
      s2 = txtXMax.getText();

      if (s1.equals(s2))
      {
//		txtXMin.setText(""+stPlotXY.dXMinimum);
		txtXMax.setText(""+stPlotXY.dXMaximum);

		sTemp = new String("A");
        sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtXMax.setText(""+stPlotXY.dXMaximum);

		  sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtXIncr)
    {
      iNumeric = 1;
      sTemp = txtXIncr.getText();
      sMessage = new String("X-Increment Value is a Numeric Field");

      s3 = txtXIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtXIncr.setText(""+stPlotXY.dXIncrement);
		if (stPlotXY.iXLog == 1) { txtYIncr.setText(""+stPlotXY.iXCycles); }
		sTemp = new String("A");
        sMessage = new String("X-Increment Value can not be 0");
	  }
	}

    // .. Y-Axis Widgets

    if (e.getSource() == txtYAxis)
    {
      stPlotXY.sYAxis = new String( txtYAxis.getText() );

      if (plotXY != null)
      {
        plotXY.setYAxis(  stPlotXY.sYAxis,
                          stPlotXY.iYLog,
	                      stPlotXY.dYMaximum,
	                      stPlotXY.dYMinimum,
	                      stPlotXY.dYIncrement,
                          stPlotXY.iYCycles );
      }

      if (plotXYZ != null)
      {
        plotXYZ.setYAxis( stPlotXY.sYAxis,
                          stPlotXY.iYLog,
	                      stPlotXY.dYMaximum,
	                      stPlotXY.dYMinimum,
	                      stPlotXY.dYIncrement,
                          stPlotXY.iYCycles );
      }

      if (plotXYH != null)
      {
        plotXYH.setYAxis( stPlotXY.sYAxis,
                          stPlotXY.iYLog,
	                      stPlotXY.dYMaximum,
	                      stPlotXY.dYMinimum,
	                      stPlotXY.dYIncrement,
                          stPlotXY.iYCycles );
      }
	}

    if (e.getSource() == txtYMin )
    {
      iNumeric = 1;
      sTemp = txtYMin.getText();
      sMessage = new String("Y-Minimum Value is a Numeric Field");

      s1 = txtYMin.getText();
      s2 = txtYMax.getText();

      if (s1.equals(s2))
      {
		txtYMin.setText(""+stPlotXY.dYMinimum);
//		txtYMax.setText(""+stPlotXY.dYMaximum);

		sTemp = new String("A");
        sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtYMin.setText(""+stPlotXY.dYMinimum);

		  sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtYMax )
    {
      iNumeric = 1;
      sTemp = txtYMax.getText();
      sMessage = new String("Y-Maximum Value is a Numeric Field");

      s1 = txtYMin.getText();
      s2 = txtYMax.getText();

      if (s1.equals(s2))
      {
//		txtYMin.setText(""+stPlotXY.dYMinimum);
		txtYMax.setText(""+stPlotXY.dYMaximum);

		sTemp = new String("A");
        sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtYMax.setText(""+stPlotXY.dYMaximum);

		  sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtYIncr)
    {
      iNumeric = 1;
      sTemp = txtYIncr.getText();
      sMessage = new String("Y-Increment Value is a Numeric Field");

      s3 = txtYIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtYIncr.setText(""+stPlotXY.dYIncrement);
		if (stPlotXY.iYLog == 1) { txtYIncr.setText(""+stPlotXY.iYCycles); }
		sTemp = new String("A");
        sMessage = new String("Y-Increment Value can not be 0");
	  }
	}

    // .. Z-Axis Widgets

    if (e.getSource() == txtZAxis)
    {
      stPlotXY.sZAxis = new String( txtZAxis.getText() );

//      if (plotXY != null)
//      {
//        plotXY.setZAxis(  stPlotXY.sZAxis,
//                          stPlotXY.iZLog,
//	                      stPlotXY.dZMaximum,
//	                      stPlotXY.dZMinimum,
//	                      stPlotXY.dZIncrement,
//                          stPlotXY.iZCycles );
//      }

      if (plotXYZ != null)
      {
        plotXYZ.setZAxis( stPlotXY.sZAxis,
                          stPlotXY.iZLog,
	                      stPlotXY.dZMaximum,
	                      stPlotXY.dZMinimum,
	                      stPlotXY.dZIncrement,
                          stPlotXY.iZCycles );
      }

      if (plotXYH != null)
      {
        plotXYH.setZAxis( stPlotXY.sZAxis,
                          stPlotXY.iZLog,
                          stPlotXY.dZMaximum,
                          stPlotXY.dZMinimum,
                          stPlotXY.dZIncrement,
                          stPlotXY.iZCycles );
      }
	}

    if (e.getSource() == txtZMin )
    {
      iNumeric = 1;
      sTemp = txtZMin.getText();
      sMessage = new String("Z-Minimum Value is a Numeric Field");

      s1 = txtZMin.getText();
      s2 = txtZMax.getText();

      if (s1.equals(s2))
      {
		txtZMin.setText(""+stPlotXY.dZMinimum);
		txtZMax.setText(""+stPlotXY.dZMaximum);

		sTemp = new String("A");
        sMessage = new String("Z-Minimum Value can not equal Z-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtZMin.setText(""+stPlotXY.dZMinimum);

		  sTemp = new String("A");
          sMessage = new String("Z-Minimum Value can not be greater than Z-Maximum Value");
		}
	  }
	}

    if (e.getSource() == txtZMax )
    {
      iNumeric = 1;
      sTemp = txtZMax.getText();
      sMessage = new String("Z-Maximum Value is a Numeric Field");

      s1 = txtZMin.getText();
      s2 = txtZMax.getText();

      if (s1.equals(s2))
      {
		txtZMin.setText(""+stPlotXY.dZMinimum);
		txtZMax.setText(""+stPlotXY.dZMaximum);

		sTemp = new String("A");
        sMessage = new String("Z-Minimum Value can not equal Z-Maximum Value");
	  }

	  if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	  {
		d1 = cmn.cmnString.stringToDouble( s1 );
		d2 = cmn.cmnString.stringToDouble( s2 );

		if (d1 > d2 )
		{
		  txtZMax.setText(""+stPlotXY.dZMaximum);

		  sTemp = new String("A");
          sMessage = new String("Z-Minimum Value can not be greater than Z-Maximum Value");
		}
	  }
	}
    if (e.getSource() == txtZIncr)
    {
      iNumeric = 1;
      sTemp = txtZIncr.getText();
      sMessage = new String("Z-Increment Value is a Numeric Field");

      s3 = txtZIncr.getText();
	  if ((s3.equals("0.0")) || (s3.equals("0")))
	  {
		txtZIncr.setText(""+stPlotXY.dYIncrement);
		if (stPlotXY.iZLog == 1) { txtZIncr.setText(""+stPlotXY.iZCycles); }
		sTemp = new String("A");
        sMessage = new String("Z-Increment Value can not be 0");
	  }
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
      }
      else
      {
        if ((e.getSource() == txtXMin ) ||
            (e.getSource() == txtXMax ) ||
            (e.getSource() == txtXIncr ))
        {
          if (e.getSource() == txtXMin )
          {
            stPlotXY.dXMinimum = cmn.cmnString.stringToDouble(txtXMin.getText());

			if (iXLog == 1)
			{
              dMin = Math.log(stPlotXY.dXMinimum)/Math.log(10.0);
              if (dMin < 0.0) { dMin = dMin - 1.0; }
              iMin = (int) dMin;
              stPlotXY.dXMinimum = Math.pow(10.0, iMin);
              txtXMin.setText(""+stPlotXY.dXMinimum);

              iMax = iMin + stPlotXY.iXCycles;
              stPlotXY.dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+stPlotXY.dXMaximum);
		    }
	      }

          if (e.getSource() == txtXMax )
            stPlotXY.dXMaximum = cmn.cmnString.stringToDouble(txtXMax.getText());

          if (e.getSource() == txtXIncr )
          {
			dTemp = cmn.cmnString.stringToDouble(txtXIncr.getText());
			if (iXLog == 0)
			  stPlotXY.dXIncrement = dTemp;
			else
			{
			  stPlotXY.iXCycles    = (int) dTemp;
			  txtXIncr.setText(""+stPlotXY.iXCycles);
		    }
//            stPlotXY.dXIncrement = cmn.cmnString.stringToDouble(txtXIncr.getText());

			if (iXLog == 1)
			{
              iMin = (int) ( Math.log(stPlotXY.dXMinimum)/Math.log(10.0) );
              iMax = iMin + stPlotXY.iXCycles;
              stPlotXY.dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+stPlotXY.dXMaximum);
		    }
		  }

          if (plotXY != null)
          {
            plotXY.setXAxis( stPlotXY.sXAxis,
                              stPlotXY.iXLog,
	                          stPlotXY.dXMaximum,
	                          stPlotXY.dXMinimum,
	                          stPlotXY.dXIncrement,
                              stPlotXY.iXCycles );
          }

          if (plotXYZ != null)
          {
            plotXYZ.setXAxis( stPlotXY.sXAxis,
                              stPlotXY.iXLog,
	                          stPlotXY.dXMaximum,
	                          stPlotXY.dXMinimum,
	                          stPlotXY.dXIncrement,
                              stPlotXY.iXCycles );
          }

          if (plotXYH != null)
          {
            plotXYH.setXAxis( stPlotXY.sXAxis,
                              stPlotXY.iXLog,
	                          stPlotXY.dXMaximum,
	                          stPlotXY.dXMinimum,
	                          stPlotXY.dXIncrement,
                              stPlotXY.iXCycles );
          }
	    }

        if ((e.getSource() == txtYMin ) ||
            (e.getSource() == txtYMax ) ||
            (e.getSource() == txtYIncr ))
        {
          if (e.getSource() == txtYMin )
          {
            stPlotXY.dYMinimum = cmn.cmnString.stringToDouble(txtYMin.getText());

			if (iYLog == 1)
			{
              dMin = Math.log(stPlotXY.dYMinimum)/Math.log(10.0);
              if (dMin < 0.0) { dMin = dMin - 1.0; }
              iMin = (int) dMin;
              stPlotXY.dYMinimum = Math.pow(10.0, iMin);
              txtYMin.setText(""+stPlotXY.dYMinimum);

              iMax = iMin + stPlotXY.iYCycles;
              stPlotXY.dYMaximum = Math.pow(10.0, iMax);
              txtYMax.setText(""+stPlotXY.dYMaximum);
		    }
		  }

          if (e.getSource() == txtYMax )
            stPlotXY.dYMaximum = cmn.cmnString.stringToDouble(txtYMax.getText());

          if (e.getSource() == txtYIncr )
          {
			dTemp = cmn.cmnString.stringToDouble(txtYIncr.getText());
			if (iYLog == 0)
			  stPlotXY.dYIncrement = dTemp;
			else
			{
			  stPlotXY.iYCycles    = (int) dTemp;
			  txtYIncr.setText(""+stPlotXY.iYCycles);
		    }

//            stPlotXY.dYIncrement = cmn.cmnString.stringToDouble(txtYIncr.getText());

			if (iYLog == 1)
			{
              iMin = (int) ( Math.log(stPlotXY.dYMinimum)/Math.log(10.0) );
              iMax = iMin + stPlotXY.iYCycles;
              stPlotXY.dYMaximum = Math.pow(10.0, iMax);
              txtYMax.setText(""+stPlotXY.dYMaximum);
		    }
		  }

          if (plotXY != null)
          {
            plotXY.setYAxis( stPlotXY.sYAxis,
                              stPlotXY.iYLog,
   	                          stPlotXY.dYMaximum,
	                          stPlotXY.dYMinimum,
	                          stPlotXY.dYIncrement,
                              stPlotXY.iYCycles );
          }

          if (plotXYZ != null)
          {
            plotXYZ.setYAxis( stPlotXY.sYAxis,
                              stPlotXY.iYLog,
   	                          stPlotXY.dYMaximum,
	                          stPlotXY.dYMinimum,
	                          stPlotXY.dYIncrement,
                              stPlotXY.iYCycles );
          }

          if (plotXYH != null)
          {
            plotXYH.setYAxis( stPlotXY.sYAxis,
                              stPlotXY.iYLog,
   	                          stPlotXY.dYMaximum,
	                          stPlotXY.dYMinimum,
	                          stPlotXY.dYIncrement,
                              stPlotXY.iYCycles );
          }
		}

        if ((e.getSource() == txtZMin ) ||
            (e.getSource() == txtZMax ) ||
            (e.getSource() == txtZIncr ))
        {
          if (e.getSource() == txtZMin )
          {
            stPlotXY.dZMinimum = cmn.cmnString.stringToDouble(txtZMin.getText());

			if (iZLog == 1)
			{
              dMin = Math.log(stPlotXY.dZMinimum)/Math.log(10.0);
              if (dMin < 0.0) { dMin = dMin - 1.0; }
              iMin = (int) dMin;
              stPlotXY.dZMinimum = Math.pow(10.0, iMin);
              txtZMin.setText(""+stPlotXY.dZMinimum);

              iMax = iMin + stPlotXY.iZCycles;
              stPlotXY.dZMaximum = Math.pow(10.0, iMax);
              txtZMax.setText(""+stPlotXY.dZMaximum);
		    }
	      }

          if (e.getSource() == txtZMax )
            stPlotXY.dZMaximum = cmn.cmnString.stringToDouble(txtZMax.getText());

          if (e.getSource() == txtZIncr )
          {
			dTemp = cmn.cmnString.stringToDouble(txtZIncr.getText());
			if (iZLog == 0)
			  stPlotXY.dZIncrement = dTemp;
			else
			{
			  stPlotXY.iZCycles    = (int) dTemp;
			  txtZIncr.setText(""+stPlotXY.iZCycles);
		    }
//            stPlotXY.dZIncrement = cmn.cmnString.stringToDouble(txtZIncr.getText());

			if (iZLog == 1)
			{
              iMin = (int) ( Math.log(stPlotXY.dZMinimum)/Math.log(10.0) );
              iMax = iMin + stPlotXY.iZCycles;
              stPlotXY.dZMaximum = Math.pow(10.0, iMax);
              txtZMax.setText(""+stPlotXY.dZMaximum);
		    }
		  }

  	      switch (iPanel)
	      {
            case _MAGNITUDE:
              getMagnitudeLegend();
              break;

            case _DATE:
              getDateLegend();
              break;

            case _DEPTH:
              getDepthLegend();
              break;
	      }
/*
          if (plotXYZ != null)
          {
            plotXYZ.setZAxis( stPlotXY.sZAxis,
                              stPlotXY.iZLog,
	                          stPlotXY.dZMaximum,
	                          stPlotXY.dZMinimum,
	                          stPlotXY.dZIncrement,
                              stPlotXY.iZCycles );
          }

          if (plotXYH != null)
          {
            plotXYH.setZAxis( stPlotXY.sZAxis,
                              stPlotXY.iZLog,
	                          stPlotXY.dZMaximum,
	                          stPlotXY.dZMinimum,
	                          stPlotXY.dZIncrement,
                              stPlotXY.iZCycles );
          }
*/
	    }
      }
    }
  }

  /** METHOD focusLostTime()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTime(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtYYYYs)
    {
      iDate    = START;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMs)
    {
      iDate    = START;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDs)
    {
	  iDate    = START;
	  iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHs)
    {
      iDate    = START;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmms)
    {
      iDate    = START;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmms.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSs)
    {
      iDate    = START;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String(txtssSs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Seconds Value is a Numeric Field");
    }

    if (e.getSource() == txtYYYYe)
    {
      iDate    = END;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Emd Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMe)
    {
      iDate    = END;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDe)
    {
      iDate    = END;
      iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHe)
    {
      iDate    = END;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmme)
    {
      iDate    = END;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmme.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSe)
    {
      iDate    = END;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String( txtssSe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Seconds Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iDate)
        {
          case START:
            setDateTime(START, tsStart);
            break;

          case END:
            setDateTime(END,   tsEnd);
            break;
	    }
      }
      else
      {
        if (cmn.cmnString.isNumeric(sTemp))
        {
          switch (iField)
          {
            case YEAR:
            case MONTH:
            case DAY:
            case HOUR:
            case MIN:
    	      id = cmn.cmnString.stringToInt(sTemp);
              break;
            case SEC:
              d  = cmn.cmnString.stringToDouble(sTemp);
              break;
 	      }
        }

        switch (iDate)
        {
          case START:
            switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMs.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDs.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHs.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmms.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	        }

	        if (!bVal)
              setDateTime(START, tsStart);
              break;

          case END:
  	        switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                  bVal = false;
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMe.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDe.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHe.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmme.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	      }

	      if (!bVal)
            setDateTime(END,   tsEnd);
            break;
 	    }
      }
    }
  }

  /* ===================================================================== *
   * ---------------------- OBSERVABLE ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String((String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
    }
  }
}

/** CLASS quake3DPlotControlFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class quake3DPlotControlFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  quake3DPlotControlFrame adaptee;

  quake3DPlotControlFrameFocusAdapter(quake3DPlotControlFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS quake3DPlotControlFrameObservable()
 *  <p> This Class will handle Observable Actions when action occurs on a
 *      frame or panel
 */

class quake3DPlotControlFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/25/2011
 *  @author  John Victorine
 */
