/*
 * @quakeTableFrame.java Version 1.1 02/11/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;


import quake.gui.quakeTable;
import quake.quakeSeisStruct;  // Seismic Data Structure

import plot.plotXYStruct;      // Plot XY Data Structure

/** Class quakeTableFrame
 *  <p> This Class will allow the user to select the status of the data
 *
 *  @version 1.1 02/11/2016
 *  @author  John R. Victorine
 */

public class quakeTableFrame extends JFrame implements ActionListener
{
  private Observable notifier = null;       // Observable notifier

  private quakeTable pTable = null;

  private JButton btnSelect = new JButton();
  private JButton btnCancel = new JButton();

  /** Constructor quakeTableFrame()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observable notifier
   */

  public quakeTableFrame(Observable notifier, quakeSeisStruct st, plotXYStruct stPo)
  {
    try
    {
      this.notifier = notifier;

      jbInit(st, stPo);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit(quakeSeisStruct st, plotXYStruct stPo) throws Exception
  {
    JPanel pnlButtons  = new JPanel();
    JPanel pnl         = new JPanel();
    JScrollPane scroll = new JScrollPane();

    this.setTitle("Select Status of Data");
    this.getContentPane().setLayout(new BorderLayout());

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnSelect.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSelect.setText("Select");
    btnSelect.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    pnl.setLayout(new BorderLayout());

    pTable = new quakeTable(st, stPo);
    scroll = pTable.getScrollPane();

    // Attach Widgets to frame

    this.getContentPane().add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnSelect,      null);
    pnlButtons.add(btnCancel,      null);

    this.getContentPane().add(pnl, BorderLayout.CENTER);
    pnl.add(scroll,                BorderLayout.CENTER);

    // Display the Frame

    this.setSize(new Dimension(550, 750));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
    notifier     = null;       // Observable notifier
    pTable       = null;
    btnSelect    = null;
    btnCancel    = null;

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return iSelectedRow - Selected Row.
   */

  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method select()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  private void select()
  {
    notifier.notifyObservers(new String("Event Selected"));
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnSelect) { select(); }
    if (event.getSource() == btnCancel)
    {
      if (notifier != null)
        notifier.notifyObservers( new String("Close Events Table Frame") );
    }
  }
}

/*
 *  @version 1.1 02/11/2016
 *  @author  John Victorine
 */
