/*
 * @quakePlotConstants.java Version 1.1 08/17/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import quake.quakePlotFilterStruct;
import quake.quakePlotFilterListStruct;

/** Class quakePlotConstants
 *  <p> This class will hold the Constants for the data plot classes
 *
 *  @version 1.1 08/17/2015
 *  @author  John R. Victorine
 */

public class quakePlotConstants
{
  public final static int _MAGNITUDE = 0;
  public final static int _DATE      = 1;
  public final static int _DEPTH     = 2;

  // -------------------------------------------------------
  // Filter possible symobls & colors
  // -------------------------------------------------------

  // Symbols Types

  public static final int OVAL      = plot.plotSymbolStruct.OVAL;
  public static final int RECTANGLE = plot.plotSymbolStruct.RECTANGLE;
  public static final int TRIANGLE  = plot.plotSymbolStruct.TRIANGLE;
  public static final int DIAMOND   = plot.plotSymbolStruct.DIAMOND;
  public static final int DELTA     = plot.plotSymbolStruct.DELTA;
  public static final int MAXIMUM   = 5;

  // Colors

  public static final int TOTAL = 43;
  public static final int COLOR = 39;
  public static final int COLORS[][] =
  {
    { 255,   0,   0 }, // "red"
    { 165,  42,  42 }, // "brown"
    { 160,  82,  45 }, // "sienna"
    { 222, 184, 135 }, // "burlywood"
    { 210, 180, 140 }, // "tan"
    { 250, 128, 114 }, // "salmon"
    { 255,  69,   0 }, // "orange red 1"
    { 255, 140,   0 }, // "dark orange"
    { 255, 165,   0 }, // "orange"
    { 255, 255, 128 }, // "middle yellow"

    { 255, 225,  25 }, // "darker yellow"
    { 255, 255,   0 }, // "yellow"
    { 173, 255,  47 }, // "green yellow"
    { 154, 205,  50 }, // "yellow green"
    { 154, 255, 154 }, // "pale green 1"
    { 128, 255, 128 }, // "light green"
    {   0, 255,   0 }, // "green"
    { 124, 252,   0 }, // "lawn green"
    {  34, 139,  34 }, // "forest green"
    { 107, 142,  35 }, // "olive drab"

    {  64, 224, 208 }, // "turquoise"
    { 127, 255, 212 }, // "aquamarine 1"
    { 102, 205, 170 }, // "aquamarine 3"
    {  69, 139, 116 }, // "aquamarine 4"
    { 128, 255, 255 }, // "middle cyan"
    {   0, 255, 255 }, // "cyan"
    { 135, 206, 235 }, // "sky blue"
    {   0, 191, 255 }, // "deep sky blue 1"
    {   0, 178, 238 }, // "deep sky blue 2"
    { 128, 128, 255 }, // "middle blue"

    {  65, 105, 225 }, // "royal blue"
    {   0,   0, 255 }, // "blue"
    { 255, 182, 193 }, // "light pink"
    { 255, 105, 180 }, // "hot pink"
    { 255,  20, 147 }, // "deep pink"
    { 255, 128, 255 }, // "light magenta"
    { 255,   0, 255 }, // "magenta"
    { 238, 130, 238 }, // "violet"
    { 140,   0, 211 }, // "dark violet"

    { 229, 229, 229 }, // "grey 90"
    { 190, 190, 190 }, // "grey"
    { 127, 127, 127 }, // "grey 50"
    {   0,   0,   0 }, // "black"
  };

  // -------------------------------------------------------
  // Filter by Earthquake Magnitude
  // -------------------------------------------------------

  public static final int    _TOTAL_MAG = 8;
  public static final double _MAG[][] = {
  //  Start   END   Red  Green Blue
    { 4.5, 10.0, 255,   0,   0 },  // > 4.5
    { 4.0,  4.5, 255, 127,   0 },  // 4.0 - 4.5
    { 3.5,  4.0, 255, 255,   0 },  // 3.5 - 4.0
    { 3.0,  3.5,   0, 255,   0 },  // 3.0 - 3.5
    { 2.5,  3.0,   0, 255, 255 },  // 2.5 - 3.0
    { 2.0,  2.5,   0,   0, 255 },  // 2.0 - 2.5
    { 1.0,  2.0, 180,   0, 255 },  // 1.0 - 2.0
    {-5.0,  1.0, 255, 182, 193 },  // < 1.0

  };

  public static final String MAG[] = {
    "> 4.5",
    "4.0 - 4.5",
    "3.5 - 4.0",
    "3.0 - 3.5",
    "2.5 - 3.0",
    "2.0 - 2.5",
    "1.0 - 2.0",
    "< 1.0",
  };

  /** Method getFilter()
   *  <p> This method will retrieve a default filter list
   */

  public static quakePlotFilterListStruct getFilter()
  {
	int    MAX      = 0;
	double DATA[][] = null;
	String NAME[]   = null;

	quakePlotFilterStruct     stFilter = null;
	quakePlotFilterListStruct st = new quakePlotFilterListStruct();

    MAX  = _TOTAL_MAG;
    DATA = _MAG;
    NAME = MAG;

    if ((DATA != null) && (NAME != null))
    {
	  for (int i=0; i<MAX; i++)
	  {
		stFilter           = new quakePlotFilterStruct();
		stFilter.sKEY      = new String( cmn.cmnString.UniqueName() + "_" + i );
		stFilter.iBy       = quake.quakePlotFilterStruct._BY_DATA;
        stFilter.dataStart = DATA[i][0]; // Starting Depth
        stFilter.dataEnd   = DATA[i][1];   // Ending Depth

        // Symbol and Color

        stFilter.iEmpty  = 1;      // Open or Filled Symbol
        stFilter.iSymbol = DIAMOND;
        stFilter.iRed    = (int) DATA[i][2]; // Red Color in RGB Setting
        stFilter.iGreen  = (int) DATA[i][3]; // Green Color in RGB Setting
        stFilter.iBlue   = (int) DATA[i][4]; // Blue Color in RGB Setting
        stFilter.symbol  = new String(NAME[i]);  // Label for the selected symbol.

	    st = quake.quakePlotFilterUtility.add( stFilter, st );
	  }
	}

	return (st);
  }

  /** Method getFilter()
   *  <p> This method will retrieve a depth filter list
   *  @param dStart = the Minimum Depth
   *  @param dEnd   = the Maximum Depth
   *  @param dIncr  = the Depth Plot Increment value
   */

  public static quakePlotFilterListStruct getFilter(
	              double dStart, double dEnd, double dIncr )
  {
	int    iDel     = 1;
	int    MAX      = 0;
	double data1    = 0.0;
	double data2    = 0.0;
	int    iColor   = 0;
	int    RGB[]    = { 255, 255, 255 };

	quakePlotFilterStruct     stFilter = null;
	quakePlotFilterListStruct st = new quakePlotFilterListStruct();

    MAX  = (int) ((dEnd - dStart)/dIncr);
    iDel = (int) ( COLOR / MAX );
    if (iDel == 0)
      iDel = 1;

    for (int i=0; i<MAX; i++)
    {
	  data1  = dStart + i * dIncr;
	  data2  = dStart + (i+1) * dIncr;
	  RGB    = getRGB( iColor );

  	  stFilter           = new quakePlotFilterStruct();
	  stFilter.sKEY      = new String( cmn.cmnString.UniqueName() + "_" + i );
	  stFilter.iBy       = quake.quakePlotFilterStruct._BY_DATA;
      stFilter.dataStart = data1; // Starting Depth
      stFilter.dataEnd   = data2;   // Ending Depth

      // Symbol and Color

      stFilter.iEmpty  = 1;      // Open or Filled Symbol
      stFilter.iSymbol = DIAMOND;
      stFilter.iRed    = RGB[0]; // Red Color in RGB Setting
      stFilter.iGreen  = RGB[1]; // Green Color in RGB Setting
      stFilter.iBlue   = RGB[2]; // Blue Color in RGB Setting
      stFilter.symbol  = new String(""+data1+" - "+data2);  // Label for the selected symbol.

	  st = quake.quakePlotFilterUtility.add( stFilter, st );

	  iColor = iColor + iDel;
	  if (iColor > COLOR) { iColor = 0; }
	}

	return (st);
  }

  /** Method getFilter()
   *  <p> This method will retrieve a date filter list
   *  @param tsStart = the Starting Date
   *  @param tsEnd   = the Ending Date
   */

  public static quakePlotFilterListStruct getFilter(
	             Timestamp tsStart, Timestamp tsEnd )
  {
	int    iDel     = 1;
	int    MAX      = 0;
	double data1    = 0.0;
	double data2    = 0.0;
	int    iColor   = 0;
	int    RGB[]    = { 255, 255, 255 };
	int    iout[]   = { 0, 0 };
    int    iYear    = 0;
    int    iMonth   = 0;
    int    iYStart  = 0;
    int    iYEnd    = 0;
    int    iMStart  = 0;
    int    iMEnd    = 0;
    int    iMonths  = 0;
    double dStart   = 0.0;
    double dEnd     = 0.0;
    double dIncr    = 0.0;
    int    iM       = 0;
    int    iY       = 0;
    int    iM2      = 0;
    int    iY2      = 0;

	quakePlotFilterStruct     stFilter = null;
	quakePlotFilterListStruct st = new quakePlotFilterListStruct();

    iout    = setDate( tsStart );
    iYStart = iout[0];
    iMStart = iout[1];

    iout    = setDate( tsEnd );
    iYEnd   = iout[0];
    iMEnd   = iout[1];

    iMonths = (iYEnd - iYStart) * 12 + (iMEnd - iMStart);
    dStart  = 0;
    dEnd    = iMonths;
    dIncr   = (double) ((int) (iMonths / 12));

    if (dIncr == 0.0) { dIncr = 1.0; }

    MAX  = (int) ((dEnd - dStart)/dIncr);
    iDel = (int) ( COLOR / MAX );
    if (iDel == 0)
      iDel = 1;

    iY   = iYStart;
    iM   = iMStart;

    iY2  = iYStart;
    iM2  = iMStart + (int) dIncr;

	if (iM2 > 12)
	{
	  iM2 = 1;
	  iY2 = iY2 + 1;
	}

    for (int i=0; i<MAX; i++)
    {
	  if (iM > 12)
	  {
		iM = 1;
		iY = iY + 1;
	  }

	  if (iM2 > 12)
	  {
	    iM2 = 1;
	    iY2 = iY2 + 1;
	  }

	  data1  = (double) (iY * 100  + iM);
	  data2  = (double) (iY2 * 100 + iM2);
	  RGB    = getRGB( iColor );

  	  stFilter           = new quakePlotFilterStruct();
	  stFilter.sKEY      = new String( cmn.cmnString.UniqueName() + "_" + i );
	  stFilter.iBy       = quake.quakePlotFilterStruct._BY_DATA;
      stFilter.dataStart = data1; // Starting Date [YYYYMM]
      stFilter.dataEnd   = data2; // Ending Date [YYYYMM]
//System.out.println(stFilter.dataStart+" "+stFilter.dataEnd);

      // Symbol and Color

      stFilter.iEmpty  = 1;      // Open or Filled Symbol
      stFilter.iSymbol = DIAMOND;
      stFilter.iRed    = RGB[0]; // Red Color in RGB Setting
      stFilter.iGreen  = RGB[1]; // Green Color in RGB Setting
      stFilter.iBlue   = RGB[2]; // Blue Color in RGB Setting
      stFilter.symbol  = new String(""+iY+"/"+iM+" - "+iY2+"/"+iM2);  // Label for the selected symbol.

	  st = quake.quakePlotFilterUtility.add( stFilter, st );

	  iColor = iColor + iDel;
	  if (iColor > COLOR) { iColor = 0; }

	  iM  = iM  + (int) dIncr;
	  iM2 = iM2 + (int) dIncr;
	}

	return (st);
  }

  /** Method setDate()
   *  <p> This method will set the Date Value into either the
   *      Start Date or End Date.
   *  @param ts    = timestamp
   */

  public static int[] setDate(Timestamp ts)
  {
	int iout[]         = { 0, 0 };
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String str         = "";
    String sDate       = "";
    String date[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;
    int    iYear       = 0;
    int    iMonth      = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

        iYear  = cmn.cmnString.stringToInt(date[0]);
	    iMonth = cmn.cmnString.stringToInt(date[1]);
      }
    }

    iout[0] = iYear;
    iout[1] = iMonth;

    return (iout);
  }

  /** Method getRGB()
   *  <p> This method will retrieve the RGB value of the
   *  @param  iRow = The Row Color
   *  @return RGB  = array of Red, Green, Blue Values
   */

  public static int[] getRGB( int iRow )
  {
	int RGB[] = { 255, 255, 255 };

    for (int i=0; i<TOTAL; i++)
    {
      if (i==iRow)
      {
	    RGB[0] = COLORS[i][0];  // Red
	    RGB[1] = COLORS[i][1];  // Green
	    RGB[2] = COLORS[i][2];  // Blue
	  }
	}

    if (iRow >= TOTAL)
    {
      RGB[0] = COLORS[TOTAL-1][0];  // Red
	  RGB[1] = COLORS[TOTAL-1][1];  // Green
	  RGB[2] = COLORS[TOTAL-1][2];  // Blue
	}

	return (RGB);
  }

  /** Method getSymbol()
   *  <p> This method will retrieve the Symbol by the row value
   *  @param  iRow   = The Row Value
   *  @return symbol = symbol identifier
   */

  public static int getSymbol( int iRow )
  {
    int symbol = iRow ;

    while (iRow > 4)
    {
	  iRow   = iRow - 5;
	  symbol = iRow;
	}

	if (symbol < 0)
    {
	  symbol = OVAL;
    }

    return (symbol);
  }
}

/*
 *  @version 1.1 08/17/2015
 *  @author  John Victorine
 */
