/*
 * @utilLandGridFile.java Version 1.1 05/24/2002
 *
 * Copyright (c) 2001 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package util;

import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import util.utilFileIO;
import util.utilLandGridStruct;
import util.utilLandGridListStruct;

/** Class utilLandGridFile
 *  <p> This Class is designed to open an ascii text file and read
 *      the contents and parse the data into the Land Grid Data List Structure.
 *
 *  @version 1.1 05/24/2002
 *  @author  John R. Victorine
 */

public class utilLandGridFile
{
  private int                    iError     = util.utilFileIO.NOERROR;
  private String                 sError     = new String("");

  private String                 sDirectory = "";
  private String                 sFilename  = "";
  private int                    iFileID    = util.utilFileIO.DIRECTORY;
  private utilFileIO             pFile      = new utilFileIO();
  private utilLandGridListStruct stList     = new utilLandGridListStruct();

 /** Construct utilLandGridFile()
   * <p> This is the Constructor for this class.
   *
   * @param iFileID    = To access the file as URL or FILE.
   * @param sDirectory = File Directory Path
   * @param sFilename  = File Name
   */

  public utilLandGridFile(int iFileID, String sDirectory, String sFilename)
  {
    this.iFileID    = iFileID;
    this.sDirectory = new String(sDirectory);
    this.sFilename  = new String(sFilename);

    stList.iCount = 0;

    countTRS();

    if ((iError == util.utilFileIO.NOERROR) && (stList.iCount > 0))
    {
      stList.stItem = new utilLandGridStruct[stList.iCount];
      getTRS();

      if (iError == util.utilFileIO.NOERROR)
      {
        getTRSLines();
      }
    }
  }

  /** Method delete()
   *  <p> This method will deallocate memory
   */

  public void delete()
  {
    sError     = null;

    sDirectory = null;
    sFilename  = null;

    if (pFile != null)
      pFile.delete();
    pFile = null;

    if (stList != null)
      stList.delete();
    stList = null;
  }

  /** Method OpenFile()
   *  <p> This method will open the LAS File for Read.
   */

  public void OpenFile()
  {
    /* Initialize the File IO Utility.
     * Open the File for READ only.
     */

    pFile.Open(iFileID, util.utilFileIO.READ, sDirectory, sFilename);
    if (pFile.getErrorID() == util.utilFileIO.NOERROR)
      pFile.Read();

    iError = pFile.getErrorID();
    sError = new String(pFile.getError());
  }

  /** Method CloseFile()
   *  <p> This method closes the LAS File
   */

  public void CloseFile()
  {
    pFile.Close();

    iError = pFile.getErrorID();
    sError = new String(pFile.getError());
  }

  /** Method countTRS()
   *  <p> This method Counts the number of Township, Range and Sections
   */

  public void countTRS()
  {
    int              iCount = 0;
    StringTokenizer  stToken;
    String           sText;
    String           sTemp;

    // Open File.

    OpenFile();

    // IF the File is Opened without error THEN

    if (iError == util.utilFileIO.NOERROR)
    {
      // Read each Line from the Land Grid File until the End Of File Indicator

      while (!pFile.Next().equals(util.utilFileIO.EOF))
      {
        sText = new String(pFile.getString());

        if (sText.length() > 0)
        {
          // Parse the line for first Item in the Line

          stToken = new StringTokenizer(sText, ",");

          // Count the Number of Tokens in the Line

          iCount  = stToken.countTokens();

          // Get the Next Token from the Line

          sTemp   = stToken.nextToken();

          // Trim the spaces from the Line

          sTemp.trim();

          // IF the Line contains the Character ">" THEN Increment the count.
          if (sTemp.equals(">"))
          {
            stList.iCount++;
          }
        }
      }

      pFile.Close();
    }
  }

  /** Method getTRS()
   *  <p> This method Counts the number of Township, Range and Sections
   */

  public void getTRS()
  {
    int              i         = 0;
    int              iRows     = -1;
    int              iCount    = 0;
    StringTokenizer  stToken;
    String           sText;
    String           sTemp;

    // Open File.

    OpenFile();

    // IF the File is Opened without error THEN

    if (iError == util.utilFileIO.NOERROR)
    {
      // Read each Line from the Land Grid File until the End Of File Indicator

      while (!pFile.Next().equals(util.utilFileIO.EOF))
      {
        sText = new String(pFile.getString());

        if (sText.length() > 0)
        {
          // Parse the line for first Item in the Line

          stToken = new StringTokenizer(sText, ",");

          // Count the Number of Tokens in the Line

          iCount  = stToken.countTokens();

          // Get the Next Token from the Line

          sTemp   = stToken.nextToken();

          // Trim the spaces from the Line

          sTemp.trim();

          // IF the Line contains the Character ">" THEN Increment the count.
          if (sTemp.equals(">"))
          {
            iRows++;

            if (iRows < stList.iCount)
            {
              stList.stItem[iRows] = new utilLandGridStruct();
              stList.stItem[iRows].sOriginal = new String(sText.trim());

              sTemp = stToken.nextToken();
              stList.stItem[iRows].sTownship = new String(sTemp.trim());
              sTemp = stToken.nextToken();
              stList.stItem[iRows].sRange    = new String(sTemp.trim());
              sTemp = stToken.nextToken();
              stList.stItem[iRows].section   = new String(sTemp.trim());
              sTemp = stToken.nextToken();
              stList.stItem[iRows].sValue    = new String(sTemp.trim());

              stList.stItem[iRows].iTotal = 0;
            }
          }
          else
          {
            if (iRows < stList.iCount)
            {
              stList.stItem[iRows].iTotal++;
            }
          }
        }
      }
    }

    pFile.Close();
  }

  /** Method getTRSLines()
   *  <p> This method will retrieve the Latitude and Longitude lines.
   */

  public void getTRSLines()
  {
    int              i         = 0;
    int              iRows     = 0;
    int              iCount    = 0;
    int              iColumns  = 0;
    StringTokenizer  stToken;
    String           sText;
    String           sTemp;
    Double           dTemp;
    int              iContinue = 0;
    double           dl;

    // Open File.

    OpenFile();

    // IF the File is Opened without error THEN

    if (iError == util.utilFileIO.NOERROR)
    {
      // Read each Line from the Land Grid File until the End Of File Indicator

      while ((!pFile.Next().equals(util.utilFileIO.EOF)) &&
             (iError == util.utilFileIO.NOERROR))
      {
        sText = new String(pFile.getString());

        if (sText.length() > 0)
        {
          // Parse the line for first Item in the Line

          stToken = new StringTokenizer(sText, ",");

          // Count the Number of Tokens in the Line

          iCount  = stToken.countTokens();

          // Get the Next Token from the Line

          sTemp   = stToken.nextToken();

          // Trim the spaces from the Line

          sTemp.trim();

          // IF the Line contains the Character ">" THEN Increment the count.
          if (sTemp.equals(">"))
          {
            iRows    = -1;
            iColumns = 0;

            for (i=0; i<stList.iCount; i++)
            {
              if (stList.stItem[i].sOriginal.equals(sText.trim()))
              {
                iRows = i;
              }
            }

            if (iRows > -1)
            {
              stList.stItem[iRows].dLongitude =
                  new double[stList.stItem[iRows].iTotal];
              stList.stItem[iRows].dLatitude =
                  new double[stList.stItem[iRows].iTotal];
            }
          }
          else
          {
            if (iRows > -1)
            {
			  iContinue = 0;

              try
              {
                dTemp = new Double(sTemp.trim());
                dl = dTemp.parseDouble(sTemp.trim());
                if (dl < 0.0) iContinue = 1;

                stList.stItem[iRows].dLongitude[iColumns] =
                    dTemp.parseDouble(sTemp.trim());
                if (stList.stItem[iRows].dLongitude[iColumns] > 0)
                {
                  stList.stItem[iRows].dLongitude[iColumns] =
                      -stList.stItem[iRows].dLongitude[iColumns];
                }
              }
              catch (NumberFormatException nfe)
              {
                iError = util.utilFileIO.ERROR;
                sError = new String("Problem Parsing Longitude Data for \n" +
                                     stList.stItem[iRows].sOriginal);
              }

              if (iError == util.utilFileIO.NOERROR)
              {
                sTemp = stToken.nextToken();
                sTemp.trim();

                try
                {
                  dTemp = new Double(sTemp.trim());
                  dl = dTemp.parseDouble(sTemp.trim());
                  if (dl < 0.0) iContinue = 1;

                  stList.stItem[iRows].dLatitude[iColumns] =
                      dTemp.parseDouble(sTemp.trim());
                }
                catch (NumberFormatException nfe)
                {
                  iError = util.utilFileIO.ERROR;
                  sError = new String("Problem Parsing Latitude Data for \n" +
                                       stList.stItem[iRows].sOriginal);
                }

                if (iContinue == 0)
                  iColumns++;
                else stList.stItem[iRows].iTotal--;
              }
            }
          }
        }
      }
    }

    pFile.Close();
  }

  /** Method getLandGridList()
   *  <p> This method will return the Land Grid List Data Structure
   *  @param stList = Land Grid List Data Structure
   */

  public utilLandGridListStruct getLandGridList() { return (stList); }

  /** METHOD getErrorID()
   *  <p> This method will return the error number.
   *  @return iError = Error Indicator
   */

  public int getErrorID() { return (iError); }

  /** METHOD getError()
   *  <p> This method will return the error string.
   *  @return sError = Error String
   */

  public String getError() { return (sError); }
}
